//---------------------------------------------------------------------------
// plotmain.cpp : rtkplot main
//
//			Copyright (C) 2007-2009 by T.TAKASU, All rights reserved.
//
// version : $Revision: 1.1 $ $Date: 2008/07/17 22:15:27 $
// history : 2008/07/14  1.0 new
//---------------------------------------------------------------------------
#include <vcl.h>
#include <vcl\inifiles.hpp>
#pragma hdrstop

#include "plotmain.h"
#include "plotopt.h"
#include "graph.h"
#include "refdlg.h"
#include "tspandlg.h"
#include "satdlg.h"
#include "aboutdlg.h"
#include "rtklib.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPlot *Plot;

#define PRGNAME		"RTKPLOT"			// program name

#define CHARDEG     "\260"              // character code of degree
#define CHARUP2     "\262"              // character code of ^2

#define INTARROW    60.0                // direction arrow interval (sec)
#define MAXTDIFF    30.0                // max differential time (sec)
#define DOPLIM      30.0                // dop view limit
#define TTOL        DTTOL               // time-differnce tolerance (sec)
#define THRESLIP    0.1                 // slip threshold of LG-jump (m)
#define CLORANGE    (TColor)0x00AAFF

#define MAX(x,y)    ((x)>(y)?(x):(y))
#define MIN(x,y)    ((x)<(y)?(x):(y))
#define SQRT(x)     ((x)<0.0?0.0:sqrt(x))

//---------------------------------------------------------------------------
extern "C" {
int showmsg(char *format,...) {return 0;}
}
//---------------------------------------------------------------------------
__fastcall TPlot::TPlot(TComponent* Owner) : TForm(Owner)
{
	AnsiString s;
	int freq[]={1,2,5,7,6};
	double ep[]={2000,1,1,0,0,0};
	Drag=0; Xn=Yn=-1; NObs=0;
	for (int i=0;i<2;i++) NRPos[i]=0;
	IndexObs=NULL; RawObs=0;
	Week=DispUpdate=PlotType=PosType=0;
	AnimCycle=1;
	for (int i=0;i<2;i++) {
		Sol[i].n=0; Sol[i].data=NULL;
	}
	Obs.n=Nav.n=Sbs.n=0;
	Obs.data=NULL; Nav.eph=NULL; Sbs.msgs=NULL;
	X0=Y0=Xc=Yc=Xs=Ys=0.0;
	for (int i=0;i<3;i++) OPos[i]=0.0;
	for (int i=0;i<2;i++) for (int j=0;j<10;j++) for (int k=0;k<3;k++) {
		RPos[i][j][k]=0.0;
	}
	Az=El=NULL;
	SolFiles[0]=new TStringList;
	SolFiles[1]=new TStringList;
	ObsFiles   =new TStringList;
	NavFiles   =new TStringList;
	Buff   =new Graphics::TBitmap;
	GraphT =new TGraph(Disp);
	
	for (int i=0;i<3;i++) {
		GraphG[i]=new TGraph(Disp);
		GraphG[i]->XLPos=0;
	}
	GraphR=new TGraph(Disp);
	GraphS=new TGraph(Disp);
	GraphT->Fit=0; GraphT->XLPos=2; GraphT->YLPos=4;
	
	for (int i=0;i<3;i++) TimeEna[i]=0;
	ObsFormat=TimeLabel=AutoScale=ShowStats=0;
	ShowLabel=ShowGLabel=1;
	ShowArrow=ShowSlip=ShowHalfC=ShowErr=ShowEph=0;
	PlotStyle=MarkSize=Origin=0;
	TimeInt=ElMask=YRange=0.0;
	TimeStart=TimeEnd=epoch2time(ep);
	DoubleBuffered=true;
	
	ObsType->Items->Clear();
	ObsType->Items->Add("ALL");
	for (int i=0;i<NFREQ;i++) ObsType->Items->Add(s.sprintf("L%d",freq[i]));
	for (int i=0;i<NFREQ;i++) ObsType->Items->Add(s.sprintf("P%d",freq[i]));
	ObsType->ItemIndex=0;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::FormCreate(TObject *Sender)
{
	::DragAcceptFiles(Handle,true);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::FormShow(TObject *Sender)
{
	TStringList *files=new TStringList;
	AnsiString s;
	int i,raw=0;
	char line[2048],*p,*q;
	
	Caption=s.sprintf("%s ver.%s",PRGNAME,VER_RTKLIB);
	LoadOpt();
	UpdateColor();
	UpdateEnable();
	UpdateLimOri(0);
	
	strcpy(line,GetCommandLine());
	
	for (i=0,p=line;*p;p=q+1) {
		if (*p==' ') {
			q=p;
			continue;
		}
		if (*p=='"') {
			if ((q=strchr(++p,'"'))) *q='\0';
		}
		else {
			if ((q=strchr(p,' '))) *q='\0';
		}
		if (++i>1) {
			if (!strcmp(p,"-r")) raw=1;
			else files->Add(p);
		}
		if (!q) break;
	}
	if (files->Count>0) {
		if (raw) ReadObs(files);
		else     ReadSol(files,0);
	}
	delete files;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DropFiles(TWMDropFiles msg)
{
	TStringList *files=new TStringList;
	int n;
	char buff[1024];
	
	if ((n=::DragQueryFile((HDROP)msg.Drop,0xFFFFFFFF,NULL,0))<=0) {
		return;
	}
	for (int i=0;i<n;i++) {
		::DragQueryFile((HDROP)msg.Drop,i,buff,sizeof(buff));
		files->Add(buff);
	}
	if (CheckObs(files->Strings[0])) {
		ReadObs(files);
	}
	else {
		ReadSol(files,0);
	}
	delete files;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuOpenSol1Click(TObject *Sender)
{
	if (!OpenSolDialog->Execute()) return;
	
	ReadSol(OpenSolDialog->Files,0);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuOpenSol2Click(TObject *Sender)
{
	if (!OpenSolDialog->Execute()) return;
	
	ReadSol(OpenSolDialog->Files,1);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuOpenObsClick(TObject *Sender)
{
	if (!OpenObsDialog->Execute()) return;
	
	ReadObs(OpenObsDialog->Files);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuOpenNavClick(TObject *Sender)
{
	if (!OpenObsDialog->Execute()) return;
	
	ReadNav(OpenObsDialog->Files);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuReloadClick(TObject *Sender)
{
	Reload();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuCloseClick(TObject *Sender)
{
	AnsiString s;
	
	ClearData();
	UpdateLimOri(0);
	UpdateLimTime(1);
	UpdateSelect();
	
	Caption=s.sprintf("%s ver.%s",PRGNAME,VER_RTKLIB);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuQuitClick(TObject *Sender)
{
	SaveOpt();
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuOptionsClick(TObject *Sender)
{
	PlotOptDialog->Left=Left+Width/2-PlotOptDialog->Width/2;
	PlotOptDialog->Top=Top+Height/2-PlotOptDialog->Height/2;
	PlotOptDialog->Plot=this;
	
	if (PlotOptDialog->ShowModal()!=mrOk) return;
	
	SaveOpt();
	UpdateColor();
	UpdateSize();
	UpdateOrigin();
	UpdateInfo();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuTimeClick(TObject *Sender)
{
	if (RawObs&&Obs.n>0) {
		TimeStart=Obs.data[0].time;
		TimeEnd  =Obs.data[Obs.n-1].time;
	}
	else if (BtnSol2->Down&&Sol[1].n>0) {
		TimeStart=Sol[1].data[0].time;
		TimeEnd  =Sol[1].data[Sol[1].n-1].time;
	}
	else if (Sol[0].n>0) {
		TimeStart=Sol[0].data[0].time;
		TimeEnd  =Sol[0].data[Sol[0].n-1].time;
	}
	for (int i=0;i<3;i++) {
		SpanDialog->TimeEna[i]=TimeEna[i];
	}
	SpanDialog->TimeStart=TimeStart;
	SpanDialog->TimeEnd  =TimeEnd;
	SpanDialog->TimeInt  =TimeInt;
	
	if (SpanDialog->ShowModal()!=mrOk) return;
	
	if (TimeEna[0]!=SpanDialog->TimeEna[0]||
		TimeEna[1]!=SpanDialog->TimeEna[1]||
		TimeEna[2]!=SpanDialog->TimeEna[2]||
		timediff(TimeStart,SpanDialog->TimeStart)!=0.0||
		timediff(TimeEnd,SpanDialog->TimeEnd)!=0.0||
		TimeInt!=SpanDialog->TimeInt) {
		
		for (int i=0;i<3;i++) TimeEna[i]=SpanDialog->TimeEna[i];
		
		TimeStart=SpanDialog->TimeStart;
		TimeEnd  =SpanDialog->TimeEnd;
		TimeInt  =SpanDialog->TimeInt;
		
		Reload();
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuToolBarClick(TObject *Sender)
{
	MenuToolBar->Checked=!MenuToolBar->Checked;
	UpdateEnable();
	UpdateSize();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuStatusBarClick(TObject *Sender)
{
	MenuStatusBar->Checked=!MenuStatusBar->Checked;
	UpdateEnable();
	UpdateSize();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuAboutClick(TObject *Sender)
{
	AboutDialog->About=PRGNAME;
	AboutDialog->IconIndex=2;
	AboutDialog->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnSol1DblClick(TObject *Sender)
{
	if (!OpenSolDialog->Execute()) return;
	ReadSol(OpenSolDialog->Files,0);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnSol2DblClick(TObject *Sender)
{
	if (!OpenSolDialog->Execute()) return;
	ReadSol(OpenSolDialog->Files,1);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnAnimateClick(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
int __fastcall TPlot::CheckObs(AnsiString file)
{
	char *p;
	
	if (!(p=strrchr(file.c_str(),'.'))) return 0;
	if (!strcmp(p,".Z")||!strcmp(p,".gz")||!strcmp(p,".zip")) {
		return *(p-1)=='o'||*(p-1)=='O'||*(p-1)=='d'||*(p-1)=='D';
	}
	return !strcmp(p,".obs")||!strcmp(p,".OBS")||
		   !strcmp(p+3,"o" )||!strcmp(p+3,"O" )||
		   !strcmp(p+3,"d" )||!strcmp(p+3,"D" );
}
//---------------------------------------------------------------------------
void __fastcall TPlot::ClearData(void)
{
	double ep[]={2000,1,1,0,0,0};
	
	for (int i=0;i<2;i++) NRPos[i]=0;
	
	free(IndexObs);
	IndexObs=NULL;
	RawObs=0;
	Week=PosType=0;
	
	for (int i=0;i<2;i++) {
		free(Sol[i].data);
		Sol[i].data=NULL;
		Sol[i].n=0;
	}
	free(Obs.data);
	free(Nav.eph );
	Obs.data=NULL;
	Nav.eph =NULL;
	Obs.n=0;
	Nav.n=0;
	
	for (int i=0;i<3;i++) OPos[i]=0.0;
	
	for (int i=0;i<2;i++) {
		for (int j=0;j<10;j++) {
			for (int k=0;k<3;k++) {
				RPos[i][j][k]=0.0;
			}
		}
	}
	free(Az);
	free(El);
	Az=NULL;
	El=NULL;
	
	SolFiles[0]->Clear();
	SolFiles[1]->Clear();
	ObsFiles->Clear();
	NavFiles->Clear();
	
	for (int i=0;i<3;i++) TimeEna[i]=0;
	
	TimeStart=TimeEnd=epoch2time(ep);
}
//---------------------------------------------------------------------------
int __fastcall TPlot::ReadSol(TStrings *files, int sel)
{
	solp_t sol={0};
	AnsiString s;
	gtime_t ts,te;
	double tint,rpos[10][3]={{0}};
	int n=0,stat,nrpos=0;
	char *paths[MAXNFILE];
	
	if (files->Count<=0) return 0;
	
	Screen->Cursor=crHourGlass;
	
	GetTimeSpan(&ts,&te,&tint);
	
	for (int i=0;i<files->Count&&n<MAXNFILE;i++) {
		paths[n++]=files->Strings[i].c_str();
	}
	stat=readsolt(paths,n,ts,te,tint,0,&sol,rpos,&nrpos,NULL,NULL);
	
	if (stat<0||sol.n<=0) {
		UpdateEnable();
		Screen->Cursor=crDefault;
		ShowMsg(s.sprintf("no solution data : %s...",paths[0]));
		ShowLegend(NULL);
		return 0;
	}
	free(Sol[sel].data); Sol[sel]=sol;
	
	for (int i=0;i<nrpos&&i<10;i++) {
		for (int j=0;j<3;j++) {
			RPos[sel][i][j]=rpos[i][j];
		}
	}
	NRPos[sel]=nrpos;
	PosType=stat;
	RawObs=0;
	
	if (SolFiles[sel]!=files) {
		SolFiles[sel]->Assign(files);
	}
	Caption="";
	
	for (int i=0;i<2;i++) {
		if (SolFiles[i]->Count==0) continue;
		Caption=Caption+SolFiles[i]->Strings[0]+(SolFiles[i]->Count>1?"... ":" ");
	}
	if (PosType==0) {
		for (int i=0;i<NRPos[sel];i++) {
			RPos[sel][i][0]*=D2R;
			RPos[sel][i][1]*=D2R;
		}
		for (int i=0;i<Sol[sel].n;i++) {
			Sol[sel].data[i].pos[0]*=D2R;
			Sol[sel].data[i].pos[1]*=D2R;
		}
		if (norm(RPos[sel][0],3)<=0.0) {
			for (int i=0;i<3;i++) RPos[sel][0][i]=Sol[sel].data[0].pos[i];
		}
	}
	else {
		NRPos[sel]=1;
		RPos[sel][0][0]=0.0;
		RPos[sel][0][1]=0.0;
		RPos[sel][0][2]=0.0;
	}
	BtnSol12->Down=False;
	if (sel==0) {
		BtnSol1->Down=true;
	}
	else {
		BtnSol2->Down=true;
	}
	if (sel==0||Sol[0].n<=0) {
		time2gpst(Sol[sel].data[0].time,&Week);
		UpdateOrigin();
		UpdateLimTime(1);
		if (AutoScale) UpdateLimFit(1);
		else		   UpdateLimOri(1);
	}
	UpdateSelect();
	
	Screen->Cursor=crDefault;
	return 1;
}
//---------------------------------------------------------------------------
int __fastcall TPlot::ReadObs(TStrings *files)
{
	obs_t obs={0};
	nav_t nav={0};
	AnsiString s;
	gtime_t ts,te;
	double tint;
	int n=0,m=0,nobs;
	char *paths[MAXNFILE],*navs[MAXNFILE],*p;
	
	if (files->Count<=0) return 0;
	
	Screen->Cursor=crHourGlass;
	
	GetTimeSpan(&ts,&te,&tint);
	
	for (int i=0;i<files->Count&&n<MAXNFILE-1;i++) {
		
		paths[n++]=files->Strings[i].c_str();
		
		if (!CheckObs(files->Strings[i])) continue;
		
		navs[m]=new char [1024];
		
		strcpy(navs[m],files->Strings[i].c_str());
		
		if (!(p=strrchr(navs[m],'.'))) {
			delete [] navs[m];
			continue;
		}
		if      (!strcmp(p,".obs")) strcpy(p,".nav");
		else if (!strcmp(p,".OBS")) strcpy(p,".NAV");
		else if (!strcmp(p+3,"o" )) strcpy(p+3,"n" );
		else if (!strcmp(p+3,"d" )) strcpy(p+3,"n" );
		else if (!strcmp(p+3,"O" )) strcpy(p+3,"N" );
		else if (!strcmp(p+3,"D" )) strcpy(p+3,"N" );
		else {
			delete [] navs[m];
			continue;
		}
		paths[n++]=navs[m++];
	}
	nobs=readrnxt(paths,n,ts,te,tint,&obs,&nav);
	
	for (int i=0;i<m;i++) {
		delete [] navs[i];
	}
	if (nobs<=0) {
		free(nav.eph);
		Screen->Cursor=crDefault;
		ShowMsg(s.sprintf("no raw obs data: %s...",paths[0]));
		ShowLegend(NULL);
		return 0;
	}
	free(Obs.data);
	free(Nav.eph );
	Obs=obs;
	Nav=nav;
	RawObs=1;
	if (ObsFiles!=files) {
		ObsFiles->Assign(files);
	}
	NavFiles->Clear();
	Caption=s.sprintf("%s%s",files->Strings[0].c_str(),files->Count>1?"...":"");
	
	delete [] IndexObs;
	IndexObs=new int [nobs+1];
	
	NObs=0;
	for (int i=0;i<Obs.n&&NObs<nobs;) {
		gtime_t time=Obs.data[i].time;
		IndexObs[NObs++]=i;
		for (i=i+1;i<Obs.n;i++) {
			if (timediff(Obs.data[i].time,time)>TTOL) break;
		}
	}
	IndexObs[NObs]=Obs.n;
	
	UpdateAzEl();
	
	BtnSol1->Down=true;
	time2gpst(Obs.data[0].time,&Week);
	TimeScroll->Position=0;
	TimeScroll->Max=NObs<=0?100:NObs-1;
	UpdateLimTime(1);
	UpdateSelect();
	
	Screen->Cursor=crDefault;
	
	return 1;
}
//---------------------------------------------------------------------------
int __fastcall TPlot::ReadNav(TStrings *files)
{
	AnsiString s;
	obs_t obs;
	gtime_t ts,te;
	double tint;
	int n=0,nobs,types[MAXNFILE];
	char *paths[MAXNFILE],*p;
	
	if (files->Count<=0) return 0;
	
	Screen->Cursor=crHourGlass;
	
	GetTimeSpan(&ts,&te,&tint);
	
	for (int i=0;i<files->Count&&n<MAXNFILE;i++) {
		paths[n++]=files->Strings[i].c_str();
	}
	readrnxt(paths,n,ts,te,tint,&obs,&Nav);
	
	if (Nav.n<=0&&Nav.ng<=0) {
		Screen->Cursor=crDefault;
		ShowLegend(NULL);
		ShowMsg(s.sprintf("no nav message: %s...",paths[0]));
		return 0;
	}
	if (NavFiles!=files) {
		NavFiles->Assign(files);
	}
	UpdateAzEl();
	UpdateSelect();
	
	Screen->Cursor=crDefault;
	
	return 1;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateAzEl(void)
{
	int i,j,ns,vsat[MAXOBS];
	
	delete [] Az;
	delete [] El;
	Az=new double[Obs.n];
	El=new double[Obs.n];
	
	for (i=0;i<Obs.n;i=j) {
		double rr[3]={0},Q[9]={0},dtr=0,azel[MAXOBS*2]={0};
		
		for (j=i;j<Obs.n;j++) {
			if (timediff(Obs.data[j].time,Obs.data[i].time)>0.0) break;
		}
		ns=pntpos(Obs.data+i,j-i,&Nav,15*D2R,0.0,rr,Q,&dtr,azel,vsat);
		
		for (int k=0;k<j-i;k++) {
			Az[i+k]=ns<4?0.0:azel[k*2];
			El[i+k]=ns<4?0.0:azel[1+k*2];
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::Reload(void)
{
	if (RawObs) {
		ReadObs(ObsFiles);
		ReadNav(NavFiles);
	}
	else {
		ReadSol(SolFiles[0],0);
		ReadSol(SolFiles[1],1);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DispPaint(TObject *Sender)
{
	TCanvas *c=Disp->Canvas;
	TRect r=Disp->ClientRect;
	
	if (DispUpdate) {
		c->Pen  ->Color=CColor[0];
		c->Brush->Color=CColor[0];
		c->Pen  ->Style=psSolid;
		c->Brush->Style=bsSolid;
		c->FillRect(r);
		
		c->Pen  ->Color=CColor[1];
		c->Brush->Style=bsClear;
		c->Rectangle(r);
		
		int level=Drag?0:1;
		
		switch (PlotType) {
			case 0: DrawGraphT(level); break;
			case 1: DrawGraphG(level); break;
			case 2: DrawGraphG(level); break;
			case 3: DrawGraphG(level); break;
			case 4: DrawRaw   (level); break;
			case 5: DrawSats  (level); break;
			case 6: DrawDop   (level); break;
		}
		Buff->SetSize(Disp->ClientWidth,Disp->ClientHeight);
		Buff->Canvas->CopyRect(r,c,r);
		DispUpdate=0;
	}
	else Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::Redraw(void)
{
	TCanvas *c=Disp->Canvas;
	TRect r=Disp->ClientRect;
	c->CopyRect(r,Buff->Canvas,r);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::Refresh(void)
{
	DispUpdate=1;
	Disp->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DispMouseDown(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
	TSpeedButton *btn[]={BtnOn1,BtnOn2,BtnOn3};
	X0=X;
	Y0=Y;
	Drag=Shift.Contains(ssLeft)?1:(Shift.Contains(ssRight)?11:0);
	
	if (PlotType==0) {
		int i=SearchPos(X,Y);
		if (i>=0) {
			TimeScroll->Position=i;
			UpdateTime();
			Drag=0;
			Refresh();
			return;
		}
		GraphT->GetCent(Xc,Yc);
		GraphT->GetScale(Xs,Ys);
		Screen->Cursor=Drag==1?crSizeAll:crVSplit;
	}
	else if (PlotType<=3) {
		int area,ind,sel;
		
		area=-1;
		ind =TimeScroll->Position;
		sel =!BtnSol1->Down&&BtnSol2->Down?1:0;
		
		if (!BtnCenterTrack->Down&&0<=ind&&ind<Sol[sel].n) {
			TPoint p;
			double xl[2],yl[2],x=TimePos(Sol[sel].data[ind].time);
			
			GraphG[0]->GetLim(xl,yl);
			GraphG[0]->ToPoint(x,yl[1],p);
			
			if ((X-p.x)*(X-p.x)+(Y-p.y)*(Y-p.y)<25) {
				Screen->Cursor=crSizeWE;
				Drag=20; Refresh();
				return;
			}
		}
		for (int i=0;i<3;i++) {
			if (!btn[i]->Down) continue;
			TPoint p(X,Y);
			
			GraphG[i]->GetCent(Xc,Yc);
			GraphG[i]->GetScale(Xs,Ys);
			area=GraphG[i]->OnAxis(p);
			
			if (Drag==1&&area==0) {
				Screen->Cursor=crSizeAll;
				Drag+=i;
				return;
			}
			else if (area==1) {
				Screen->Cursor=Drag==1?crSizeNS:crVSplit;
				Drag+=i+4;
				return;
			}
			else if (area==0) break;
		}
		if (area==0||area==8) {
			Screen->Cursor=Drag==1?crSizeWE:crHSplit;
			Drag+=3;
		}
		else Drag=0;
	}
	else if (PlotType==4||PlotType==6) {
		int area,ind=TimeScroll->Position;
		
		if (!BtnCenterTrack->Down&&0<=ind&&ind<NObs) {
			TPoint p;
			double xl[2],yl[2],x=TimePos(Obs.data[IndexObs[ind]].time);
			
			GraphR->GetLim(xl,yl);
			GraphR->ToPoint(x,yl[1],p);
			
			if ((X-p.x)*(X-p.x)+(Y-p.y)*(Y-p.y)<25) {
				Screen->Cursor=crSizeWE; Drag=20; Refresh();
				return;
			}
		}
		TPoint p(X,Y);
		GraphR->GetCent(Xc,Yc);
		GraphR->GetScale(Xs,Ys);
		area=GraphR->OnAxis(p);
		
		if (area==0||area==8) {
			Screen->Cursor=Drag==1?crSizeWE:crHSplit;
			Drag+=3;
		}
		else {
			Drag=0;
		}
	}
	else Drag=0;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DispMouseLeave(TObject *Sender)
{
	Xn=Yn=-1;
	Panel22->Visible=false;
	Message2->Caption="";
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DispMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
	if (X!=Xn||Y!=Yn) {Xn=X; Yn=Y;} else return;
	
	double x,y,xs,ys,dx,dy;
	double dxs,dys;
	
	dx=(X0-X)*Xs;
	dy=(Y-Y0)*Ys;
	dxs=pow(2.0,(X0-X)/100.0);
	dys=pow(2.0,(Y-Y0)/100.0);
	
	if (Drag==0) {
		UpdatePoint(X,Y);
		return;
	}
	if (PlotType==0) {
		if (Drag==1) {
			GraphT->SetCent(Xc+dx,Yc+dy);
			BtnCenterTrack->Down=false;
		}
		else GraphT->SetScale(Xs*dys,Ys*dys);
	}
	else if (PlotType<=3) {
		if (Drag<=4) {
			for (int i=0;i<3;i++) {
				GraphG[i]->GetCent(x,y);
				GraphG[i]->SetCent(Xc+dx,i==Drag-1?Yc+dy:y);
				BtnCenterTrack->Down=false;
			}
		}
		else if (Drag<=7) {
			double x,y;
			GraphG[Drag-5]->GetCent(x,y);
			GraphG[Drag-5]->SetCent(x,Yc+dy);
			BtnCenterTrack->Down=false;
		}
		else if (Drag<=14) {
			for (int i=0;i<3;i++) {
				double xs,ys;
				GraphG[i]->GetScale(xs,ys);
				GraphG[i]->SetScale(Xs*dxs,ys);
			}
		}
		else if (Drag<=17) {
			GraphG[Drag-15]->GetScale(xs,ys);
			GraphG[Drag-15]->SetScale(xs,Ys*dys);
		}
		else if (Drag==20) {
			int i,sel=!BtnSol1->Down&&BtnSol2->Down?1:0;
			TPoint p(X,Y);
			GraphG[0]->ToPos(p,x,y);
			for (i=0;i<Sol[sel].n;i++) {
				if (TimePos(Sol[sel].data[i].time)>=x) break;
			}
			TimeScroll->Position=i;
			Refresh();
			return;
		}
		GraphG[0]->GetCent(x,y); GraphG[0]->GetScale(xs,ys);
	}
	else if (PlotType==4||PlotType==6) {
		
		if (Drag<=4) {
			GraphR->GetCent(x,y);
			GraphR->SetCent(Xc+dx,Yc+dy);
			BtnCenterTrack->Down=false;
		}
		else if (Drag<=14) {
			double xs,ys;
			GraphR->GetScale(xs,ys);
			GraphR->SetScale(Xs*dxs,ys);
		}
		else if (Drag==20) {
			int i;
			TPoint p(X,Y);
			GraphR->ToPos(p,x,y);
			for (i=0;i<NObs;i++) if (TimePos(Obs.data[IndexObs[i]].time)>=x) break;
			TimeScroll->Position=i;
			Refresh();
			return;
		}
	}
	else return;
	
	BtnCenterOri->Down=false;
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DispMouseUp(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
	Drag=0; Screen->Cursor=crDefault;
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::FormMouseWheel(TObject *Sender, TShiftState Shift,
      int WheelDelta, TPoint &MousePos, bool &Handled)
{
	double xs,ys,ds=pow(2.0,-WheelDelta/1200.0);
	
	Handled=true;
	if (Xn<0||Yn<0) return;
	
	if (PlotType==0) {
		GraphT->GetScale(xs,ys);
		GraphT->SetScale(xs*ds,ys*ds);
	}
	else if (PlotType<=3) {
		TPoint p(Xn,Yn);
		int area;
		
		for (int i=0;i<3;i++) {
			area=GraphG[i]->OnAxis(p);
			if (area==0||area==1||area==2) {
				GraphG[i]->GetScale(xs,ys);
				GraphG[i]->SetScale(xs,ys*ds);
			}
			else if (area==0) break;
		}
		if (area==8) {
			for (int i=0;i<3;i++) {
				GraphG[i]->GetScale(xs,ys);
				GraphG[i]->SetScale(xs*ds,ys);
			}
		}
		GraphG[0]->GetScale(xs,ys); GraphR->SetScale(xs,ys);
	}
	else if (PlotType==4||PlotType==6) {
		TPoint p(Xn,Yn);
		int area=GraphR->OnAxis(p);
		if (area==0||area==8) {
			GraphR->GetScale(xs,ys);
			GraphR->SetScale(xs*ds,ys);
		}
	}
	else return;
	
	BtnCenterOri->Down=false;
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnSol1Click(TObject *Sender)
{
	BtnSol12->Down=false;
	UpdateSelect();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnSol2Click(TObject *Sender)
{
	BtnSol12->Down=false;
	UpdateSelect();	
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnSol12Click(TObject *Sender)
{
	BtnSol1->Down=false; BtnSol2->Down=false;
	UpdateSelect();	
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnCenterOriClick(TObject *Sender)
{
	if (PlotType<=4) BtnCenterTrack->Down=false;
	UpdateLimOri(0);
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnFitXClick(TObject *Sender)
{
	BtnCenterTrack->Down=false;
	if (PlotType!=0) UpdateLimTime(0); else UpdateLimFit(0);
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnFitYClick(TObject *Sender)
{
	if (PlotType==0) BtnCenterTrack->Down=false;
	UpdateLimFit(0);
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnShowTrackClick(TObject *Sender)
{
	UpdateEnable();
	UpdateInfo();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnCenterTrackClick(TObject *Sender)
{
	if (PlotType<=4||PlotType==6) Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuCenterOriClick(TObject *Sender)
{
	if (PlotType<=4) BtnCenterTrack->Down=false;
	UpdateLimOri(0);
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuFitXClick(TObject *Sender)
{
	BtnCenterTrack->Down=false;
	if (PlotType!=0) UpdateLimTime(0); else UpdateLimFit(0);
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuFitYClick(TObject *Sender)
{
	if (PlotType==0) BtnCenterTrack->Down=false;
	UpdateLimFit(0);
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuShowTrackClick(TObject *Sender)
{
	MenuShowTrack->Checked=!MenuShowTrack->Checked;
	BtnShowTrack->Down=MenuShowTrack->Checked;
	UpdateEnable();
	UpdateInfo();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuCenterTrackClick(TObject *Sender)
{
	MenuCenterTrack->Checked=!MenuCenterTrack->Checked;
	BtnCenterTrack->Down=MenuCenterTrack->Checked;
	if (PlotType<=4||PlotType==6) Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnReloadClick(TObject *Sender)
{
	Reload();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuSatClick(TObject *Sender)
{
	SatDialog->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuAnimStartClick(TObject *Sender)
{
	if (!BtnAnimate->Enabled) return;
	BtnAnimate->Down=true;
	UpdateEnable();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::MenuAnimStopClick(TObject *Sender)
{
	if (!BtnAnimate->Enabled) return;
	BtnAnimate->Down=false;
	UpdateEnable();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::TimerTimer(TObject *Sender)
{
	int cycle=AnimCycle<=0?1:AnimCycle;
	
	if (!BtnAnimate->Enabled||!BtnAnimate->Down) return;
	
	if (TimeScroll->Position+cycle<=TimeScroll->Max) {
		TimeScroll->Position=TimeScroll->Position+cycle;
	}
	else TimeScroll->Position=TimeScroll->Max;
	if (TimeScroll->Position==TimeScroll->Max) BtnAnimate->Down=false;
	
	UpdateEnable();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::PlotTypeSolChange(TObject *Sender)
{
	PlotType=PlotTypeSol->ItemIndex;
	UpdateSize();
	if (AutoScale) UpdateLimFit(0);
	UpdateEnable();
	UpdateInfo();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::PlotTypeObsChange(TObject *Sender)
{
	PlotType=PlotTypeObs->ItemIndex+4;
	UpdateSize();
	if (AutoScale) UpdateLimFit(0);
	UpdateEnable();
	UpdateInfo();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::QFlagChange(TObject *Sender)
{
	QFlag->ItemIndex;
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::ObsTypeChange(TObject *Sender)
{
	UpdateInfo();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DopTypeChange(TObject *Sender)
{
	UpdateInfo();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::TimeScrollChange(TObject *Sender)
{
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::FormResize(TObject *Sender)
{
	UpdateSize();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnOn1Click(TObject *Sender)
{
	UpdateSize();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnOn2Click(TObject *Sender)
{
	UpdateSize();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::BtnOn3Click(TObject *Sender)
{
	UpdateSize();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::GetTimeSpan(gtime_t *ts, gtime_t *te, double *tint)
{
	gtime_t t0={0};
	*ts=*te=t0; *tint=0.0;
	if (TimeEna[0]) *ts=TimeStart;
	if (TimeEna[1]) *te=TimeEnd;
	if (TimeEna[2]) *tint=TimeInt;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawGraphT(int level)
{
	AnsiString label;
	TPoint p1,p2;
	TColor color;
	gtime_t *t;
	double xt,yt,sx,sy,xx,yy,zz,xxs,yys,xxyys,*x,*y,*z,*xs,*ys,*xys,dir=0.0,vel;
	int qq,*q,ind,sel,n,m;
		
	ind=TimeScroll->Position;
	sel=!BtnSol1->Down&&BtnSol2->Down?1:0;
	
	if (BtnShowTrack->Down&&0<=ind&&ind<Sol[sel].n) {
		SolToXyz(Sol[sel].data+ind,1,NULL,&xx,&yy,&zz,&qq,&xxs,&yys,NULL,&xxyys,0,0);
		if (BtnCenterTrack->Down) GraphT->SetCent(xx,yy);
	}
	GraphT->DrawAxis(ShowLabel,ShowGLabel);
	
	if (ShowCompass) { // compass
		GraphT->GetPos(p1,p2);
		p1.x+=40;
		p1.y+=60;
		GraphT->DrawMark(p1,13,CColor[2],50,0);
	}
	if (ShowScale) { // scale
		GraphT->GetPos(p1,p2);
		GraphT->GetTick(xt,yt);
		GraphT->GetScale(sx,sy);
		p2.x-=70;
		p2.y-=25;
		GraphT->DrawMark(p2,11,CColor[2],(int)(xt/sx+0.5),0);
		p2.y-=3;
		if      (xt<0.01  ) label.sprintf("%.0f mm",xt*1000.0);
		else if (xt<1.0   ) label.sprintf("%.0f cm",xt*100.0);
		else if (xt<1000.0) label.sprintf("%.0f m" ,xt);
		else                label.sprintf("%.0f km",xt/1000.0);
		GraphT->DrawText(p2,label,CColor[2],0,1,0);
	}
	if (BtnSol12->Down) {
		DrawGraphTD(level);
		return;
	}
	for (int i=0,p=0;i<2;i++) {
		if ((i==0&&!BtnSol1->Down)||(i==1&&!BtnSol2->Down)) continue;
		if ((n=Sol[i].n)<=0) continue;
		t  =new gtime_t[n];
		x  =new double [n];
		y  =new double [n];
		z  =new double [n];
		xs =new double [n];
		ys =new double [n];
		xys=new double [n];
		q  =new int    [n];
		
		m=SolToXyz(Sol[i].data,n,t,x,y,z,q,xs,ys,NULL,xys,QFlag->ItemIndex,0);
		
		if (level&&ShowArrow) DrawArrow(t,x,y,m);
		
		if (level&&ShowErr&&xs&&PlotStyle<=1) { // error circle
			DrawGraphTC(x,y,z,xs,ys,xys,m,i);
		}
		if (!level||!(PlotStyle%2)) {
			GraphT->DrawPoly(x,y,m,CColor[3],i);
		}
		if (level&&PlotStyle<2) {
			for (int j=0;j<m;j++) {
				GraphT->DrawMark(x[j],y[j],i,MColor[q[j]],MarkSize,0);
			}
		}
		if (ShowStats) DrawStatTC(x,y,z,m,p++);
		
		delete [] t;
		delete [] x;
		delete [] y;
		delete [] z;
		delete [] xs;
		delete [] ys;
		delete [] xys;
		delete [] q;
		
		for (int j=0;j<NRPos[i];j++) { // reference pos
			double xr,yr,zr;
			solpd_t sol={{0}};
			for (int k=0;k<3;k++) sol.pos[k]=RPos[i][j][k];
			
			SolToXyz(&sol,1,NULL,&xr,&yr,&zr,NULL,NULL,NULL,NULL,NULL,0,0);
			
			label.sprintf("%s %d",j==0?"Ref Pos":"Slave Pos",i+1);
			
			GraphT->DrawMark(xr,yr,0,CColor[2],MarkSize,0);
			GraphT->DrawText(xr,yr,label,CColor[2],0,2,0);
		}
	}
	if (!level) { // center +
		GraphT->GetPos(p1,p2);
		p1.x=(p1.x+p2.x)/2;
		p1.y=(p1.y+p2.y)/2;
		GraphT->DrawMark(p1,5,CColor[1],20,0);
	}
	if (BtnShowTrack->Down&&0<=ind&&ind<Sol[sel].n) {
		
		if (ShowErr) { // error circle
			DrawGraphTC(&xx,&yy,&zz,&xxs,&yys,&xxyys,1,0);
		}
		// current mark
		GraphT->DrawMark(xx,yy,0,CColor[0],MarkSize*2+10,0);
		GraphT->DrawMark(xx,yy,1,CColor[2],MarkSize*2+10,0);
		
		if (ind>0&&ShowArrow) { // direction arrow
			vel=SolToVel(Sol[sel].data[ind].time,Sol[sel].data[ind-1].time,
						 Sol[sel].data[ind].pos, Sol[sel].data[ind-1].pos,dir);
			
			if (vel>=1.0&&PosType==0) {
				GraphT->DrawMark(xx,yy,10,CColor[2],MarkSize*2+30,90-(int)dir);
			}
			else {
				GraphT->DrawMark(xx,yy,5,CColor[2],MarkSize*2+14,0);
			}
		}
		else {
			GraphT->DrawMark(xx,yy,5,CColor[2],MarkSize*2+14,0);
		}
		// current mark
		color=MColor[Sol[sel].data[ind].posq];
		GraphT->DrawMark(xx,yy,0,CColor[2],MarkSize*2+6,0);
		GraphT->DrawMark(xx,yy,0,color,MarkSize*2+4,0);
		
		if (PosType) {
			solpd_t sol={0};
			double x[2],y[2],z[2];
			
			x[0]=xx;
			y[0]=yy;
			SolToXyz(&sol,1,NULL,x+1,y+1,z+1,NULL,NULL,NULL,NULL,NULL,0,0);
			
			GraphT->DrawPoly(x,y,2,CColor[3],1);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawGraphTD(int level)
{
	gtime_t *t1,*t2;
	double *x1,*y1,*z1,*x2,*y2,*z2,tt;
	int n1=Sol[0].n,n2=Sol[1].n,m=0,m1,m2,*q1,*q2;
	
	if (n1==0||n2==0) return;
	
	t1=new gtime_t[n1];
	t2=new gtime_t[n2];
	x1=new double [n1];
	y1=new double [n1];
	z1=new double [n1];
	x2=new double [n2];
	y2=new double [n2];
	z2=new double [n2];
	q1=new int    [n1];
	q2=new int    [n2];
	
	m1=SolToXyz(Sol[0].data,n1,t1,x1,y1,z1,q1,NULL,NULL,NULL,NULL,0,0);
	m2=SolToXyz(Sol[1].data,n2,t2,x2,y2,z2,q2,NULL,NULL,NULL,NULL,0,0);
	
	for (int i=0,j=0;i<m1&&j<m2;i++,j++) {
		tt=timediff(t1[i],t2[j]);
		if (tt<-TTOL) {j--; continue;} else if (tt>TTOL) {i--; continue;}
		x1[m]=x1[i]-x2[j];
		y1[m]=y1[i]-y2[j];
		z1[m]=z1[i]-z2[j];
		q1[m]=q1[i]>q2[j]?q1[i]:q2[j];
		if (QFlag->ItemIndex==0||q1[m]==QFlag->ItemIndex) m++;
	}
	if (!level||!(PlotStyle%2)) {
		GraphT->DrawPoly(x1,y1,m,CColor[3],0);
	}
	if (level&&PlotStyle<2) {
		for (int i=0;i<m;i++) {
			GraphT->DrawMark(x1[i],y1[i],0,MColor[q1[i]],MarkSize,0);
		}
	}
	if (ShowStats) DrawStatTC(x1,y1,z1,m,0);
	
	delete [] t1;
	delete [] t2;
	delete [] x1;
	delete [] y1;
	delete [] z1;
	delete [] x2;
	delete [] y2;
	delete [] z2;
	delete [] q1;
	delete [] q2;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawGraphTC(double *x, double *y, double *z,
	double *xs, double *ys, double *xys, int m, int style)
{
	const double sint[36]={
		 0.0000, 0.1736, 0.3420, 0.5000, 0.6428, 0.7660, 0.8660, 0.9397, 0.9848,
		 1.0000, 0.9848, 0.9397, 0.8660, 0.7660, 0.6428, 0.5000, 0.3420, 0.1736,
		 0.0000,-0.1736,-0.3420,-0.5000,-0.6428,-0.7660,-0.8660,-0.9397,-0.9848,
		-1.0000,-0.9848,-0.9397,-0.8660,-0.7660,-0.6428,-0.5000,-0.3420,-0.1736
	};
	double xc[37],yc[37],a,b,s,c;
	
	for (int i=0;i<m;i++) {
		if (xs[i]<0.0||ys[i]<=0.0) continue;
		
		a=(xys[i]<0.0?-1.0:1.0)*xys[i]*xys[i]/ys[i];
		
		if ((b=xs[i]*xs[i]-a*a)>=0.0) b=SQRT(b); else continue;
		
		for (int j=0;j<37;j++) {
			s=sint[j%36];
			c=sint[(45-j)%36];
			xc[j]=x[i]+ys[i]*c;
			yc[j]=y[i]+a*c+b*s;
		}
		GraphT->DrawPoly(xc,yc,37,CColor[1],style);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawStatTC(double *x, double *y, double *z, int m,
	int p)
{
	TPoint p1,p2;
	double *d,ave[4],std[4],rms[4],r[3],rr[3],pos[3];
	int n=0,fonth=(int)(Disp->Font->Size*1.5);
	
	d=new double[m];
	
	for (int i=0;i<m;i++) d[i]=SQRT(x[i]*x[i]+y[i]*y[i]);
	
	CalcStats(x,m,0.0,ave[0],std[0],rms[0]);
	CalcStats(y,m,0.0,ave[1],std[1],rms[1]);
	CalcStats(z,m,0.0,ave[2],std[2],rms[2]);
	CalcStats(d,m,0.0,ave[3],std[3],rms[3]);
	
	delete [] d;
	
	pos2ecef(OPos,r);
	enu2ecef(OPos,ave,rr);
	for (int i=0;i<3;i++) r[i]+=rr[i];
	ecef2pos(r,pos);
	AnsiString s[6];
	
	if (PosType==0) {
		s[n++].sprintf("ORI=%.8f" CHARDEG " %.8f" CHARDEG " %.3fm",OPos[0]*R2D,
					   OPos[1]*R2D,OPos[2]);
	}
	else {
		s[n++].sprintf("ORI=E:%.3fm N:%.3fm U:%.3fm",OPos[0],OPos[1],OPos[2]);
	}
	s[n++].sprintf("AVE=E:%6.3fm N:%6.3fm U:%6.3fm",ave[0],ave[1],ave[2]);
	s[n++].sprintf("STD=E:%6.3fm N:%6.3fm U:%6.3fm",std[0],std[1],std[2]);
	s[n++].sprintf("RMS=E:%6.3fm N:%6.3fm U:%6.3fm 2D:%6.3fm",
				   rms[0],rms[1],rms[2],2.0*rms[3]);
	
	GraphT->GetPos(p1,p2);
	p1.x=p2.x-10;
	p1.y+=8+fonth*4*p;
	
	for (int i=p==0?0:1;i<n;i++,p1.y+=fonth) {
		GraphT->DrawText(p1,s[i],CColor[2],2,2,0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawGraphG(int level)
{
	AnsiString label[]={"E-W","N-S","U-D"},unit[]={"m","m/s","m/s" CHARUP2};
	TSpeedButton *btn[]={BtnOn1,BtnOn2,BtnOn3};
	TPoint p1,p2;
	gtime_t tt,*t;
	double xyz[3],xc,yc,*x,*y,*z,*xs,*ys,*zs,xl[2],yl[2];
	int ind,sel,type=PlotType-1,qq,*q;
	
	ind=TimeScroll->Position;
	sel=!BtnSol1->Down&&BtnSol2->Down?1:0;
	
	if (0<=ind&&ind<Sol[sel].n&&BtnShowTrack->Down&&BtnCenterTrack->Down) {
		
		SolToXyz(Sol[sel].data+ind,1,&tt,xyz,xyz+1,xyz+2,NULL,NULL,NULL,NULL,NULL,0,0);
		
		for (int i=0;i<3;i++) {
			GraphG[i]->GetCent(xc,yc);
			GraphG[i]->SetCent(TimePos(tt),PlotType==1?xyz[i]:yc);
		}
	}
	int j=-1;
	for (int i=0;i<3;i++) if (btn[i]->Down) j=i;
	for (int i=0;i<3;i++) {
		if (!btn[i]->Down) continue;
		GraphG[i]->XLPos=TimeLabel?(i==j?6:5):(i==j?1:0);
		GraphG[i]->Week=Week;
		GraphG[i]->DrawAxis(ShowLabel,ShowLabel);
		GraphG[i]->GetPos(p1,p2);
		p1.x+=5;
		p1.y+=3;
		GraphG[i]->DrawText(p1,label[i]+" ("+unit[PlotType-1]+")",CColor[2],1,2,0);
	}
	if (BtnSol12->Down) {
		DrawGraphGD(level);
		return;
	}
	for (int i=0,p=0;i<2;i++) {
		int n,m;
		if ((n=Sol[i].n)<=0) continue;
		if (i==0&&!BtnSol1->Down||i==1&&!BtnSol2->Down) continue;
		
		t =new gtime_t[n];
		x =new double[n];
		y =new double[n];
		z =new double[n];
		xs=new double[n];
		ys=new double[n];
		zs=new double[n];
		q =new int[n];
		
		m=SolToXyz(Sol[i].data,n,t,x,y,z,q,xs,ys,zs,NULL,QFlag->ItemIndex,type);
		
		DrawGraphGC(t,x,y,z,q,xs,ys,zs,m,level,i);
		
		if (ShowStats) DrawStatGC(x,y,z,m,unit[PlotType-1],p++);
		
		delete [] t;
		delete [] x;
		delete [] y;
		delete [] z;
		delete [] xs;
		delete [] ys;
		delete [] zs;
		delete [] q;
	}
	if (BtnShowTrack->Down&&0<=ind&&ind<Sol[sel].n&&(BtnSol1->Down||BtnSol2->Down)) {
		
		SolToXyz(Sol[sel].data+ind,1,&tt,xyz,xyz+1,xyz+2,&qq,NULL,NULL,NULL,NULL,0,0);
		
		for (int j=0,k=0;j<3;j++) {
			if (!btn[j]->Down) continue;
			
			GraphG[j]->GetLim(xl,yl);
			xl[0]=xl[1]=TimePos(tt);
			
			GraphG[j]->DrawPoly(xl,yl,2,ind==0?CColor[1]:CColor[2],0);
			
			if (0<=ind&&PlotType==1&&btn[j]->Down) {
			    GraphG[j]->DrawMark(xl[0],xyz[j],1,CColor[2 ],MarkSize*2+2,0);
			    GraphG[j]->DrawMark(xl[0],xyz[j],0,MColor[qq],MarkSize*2  ,0);
			}
			if (k++==0) {
				GraphG[j]->DrawMark(xl[0],yl[1]-1E-6,0,CColor[2],3,0);
				GraphG[j]->DrawMark(xl[0],yl[1]-1E-6,1,CColor[2],9,0);
			}
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawGraphGD(int level)
{
	gtime_t *t1,*t2;
	double *x1,*y1,*z1,*x2,*y2,*z2,tt;
	int n1=Sol[0].n,n2=Sol[1].n,m=0,m1,m2,*q1,*q2;
	AnsiString unit[]={"m","m/s","m/s" CHARUP2};
	
	if (n1<=0||n2<=0) return;
	
	t1=new gtime_t[n1];
	t2=new gtime_t[n2];
	x1=new double [n1];
	y1=new double [n1];
	z1=new double [n1];
	x2=new double [n2];
	y2=new double [n2];
	z2=new double [n2];
	q1=new int    [n1];
	q2=new int    [n2];
	
	m1=SolToXyz(Sol[0].data,n1,t1,x1,y1,z1,q1,NULL,NULL,NULL,NULL,0,PlotType-1);
	m2=SolToXyz(Sol[1].data,n2,t2,x2,y2,z2,q2,NULL,NULL,NULL,NULL,0,PlotType-1);
	
	for (int i=0,j=0;i<m1&&j<m2;i++,j++) {
		tt=timediff(t1[i],t2[j]);
		if (tt<-TTOL) {j--; continue;} else if (tt>TTOL) {i--; continue;}
		t1[m]=t1[i];
		x1[m]=x1[i]-x2[j];
		y1[m]=y1[i]-y2[j];
		z1[m]=z1[i]-z2[j];
		q1[m]=q1[i]>q2[j]?q1[i]:q2[j];
		if (QFlag->ItemIndex==0||q1[m]==QFlag->ItemIndex) m++;
	}
	DrawGraphGC(t1,x1,y1,z1,q1,NULL,NULL,NULL,m,level,0);
	
	if (ShowStats) {
		DrawStatGC(x1,y1,z1,m,unit[PlotType-1],0);
	}
	delete [] t1;
	delete [] t2;
	delete [] x1;
	delete [] y1;
	delete [] z1;
	delete [] x2;
	delete [] y2;
	delete [] z2;
	delete [] q1;
	delete [] q2;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawGraphGC(gtime_t *t, double *x, double *y,
	double *z, int *q, double *xs, double *ys, double *zs, int m, int level,
	int style)
{
	TSpeedButton *btn[]={BtnOn1,BtnOn2,BtnOn3};
	int hh;
	double *s[]={x,y,z},*ss[]={ys,xs,zs},*tt=new double[m];
	
	for (int i=0;i<m;i++) tt[i]=TimePos(t[i]);
	
	for (int i=0;i<3;i++) {
		if (!btn[i]->Down) continue;
		
		if (!level||!(PlotStyle%2)) {
			GraphG[i]->DrawPoly(tt,s[i],m,CColor[3],style);
		}
		if (level&&ShowErr&&ss[i]&&PlotType<=1&&PlotStyle<2) {
			double xsc,ysc;
			
			GraphG[i]->GetScale(xsc,ysc);
			for (int j=0;j<m;j++)
				GraphG[i]->DrawMark(tt[j],s[i][j],12,CColor[1],(int)(ss[i][j]*2.0/ysc),0);
		}
		if (level&&PlotStyle<2) {
			for (int j=0;j<m;j++) {
				GraphG[i]->DrawMark(tt[j],s[i][j],style,MColor[q[j]],MarkSize,0);
			}
		}
	}
	delete [] tt;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawStatGC(double *x, double *y, double *z, int m,
	AnsiString unit, int p)
{
	TSpeedButton *btn[]={BtnOn1,BtnOn2,BtnOn3};
	double *s[]={x,y,z};
	int fonth=(int)(Disp->Font->Size*1.5),j=0;
	
	for (int i=0;i<3;i++) {
		if (!btn[i]->Down) continue;
		double ave,std,rms;
		
		CalcStats(s[i],m,0.0,ave,std,rms);
		AnsiString str;
		TPoint p1,p2;
		
		GraphG[i]->GetPos(p1,p2); p1.x=p2.x-5; p1.y+=3+fonth*p;
		
		if (j==0&&p==0&&PlotType==1) {
			if (PosType==0) {
				str.sprintf("ORI=%12.8f" CHARDEG " %13.8f" CHARDEG " %8.3fm",
							OPos[0]*R2D,OPos[1]*R2D,OPos[2]);
			}
			else {
				str.sprintf("ORI=E:%6.3fm N:%6.3fm U:%6.3fm",OPos[0],OPos[1],OPos[2]);
			}
			GraphG[i]->DrawText(p1,str,CColor[2],2,2,0);
			j++; p1.y+=fonth;
		}
		char *u=unit.c_str();
		str.sprintf("AVE=%.3f%s STD=%.3f%s RMS=%.3f%s",ave,u,std,u,rms,u);
		GraphG[i]->DrawText(p1,str,CColor[2],2,2,0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawRaw(int level)
{
	AnsiString label;
	TPoint p1,p2,p;
	gtime_t time;
	obsd_t *obs;
	double xs,ys,xt,xl[2],yl[2],tt[MAXSAT]={0},xp,xc,yc,yp[MAXSAT]={0};
	int m=0,sats[MAXSAT]={0},ind=TimeScroll->Position,prn,color;
	char id[16];
	
	for (int i=0;i<Obs.n;i++) {
		if (!(satsys(Obs.data[i].sat,&prn)&NavSys)) continue;
		sats[Obs.data[i].sat-1]=1;
	}
	for (int i=0;i<MAXSAT;i++) if (sats[i]) m++;
	
	GraphR->XLPos=TimeLabel?6:1; 
	GraphR->YLPos=0;
	GraphR->Week=Week;
	GraphR->GetLim(xl,yl);
	yl[0]=0.5;
	yl[1]=m>0?m+0.5:m+10.5;
	GraphR->SetLim(xl,yl);
	GraphR->SetTick(0.0,1.0);
	
	if (0<=ind&&ind<NObs&&BtnShowTrack->Down&&BtnCenterTrack->Down) {
		xp=TimePos(Obs.data[IndexObs[ind]].time);
		GraphR->GetCent(xc,yc);
		GraphR->SetCent(xp,yc);
	}
	GraphR->DrawAxis(1,1);
	GraphR->GetPos(p1,p2);
	
	for (int i=0,j=0;i<MAXSAT;i++) {
		if (!sats[i]) continue;
		p.x=p1.x;
		p.y=p1.y+(int)((p2.y-p1.y)*(j+0.5)/m);
		yp[i]=m-(j++);
		satno2id(i+1,id);
		label=id;
		GraphR->DrawText(p,label,CColor[2],2,0,0);
	}
	p1.x=Disp->Font->Size;
	p1.y=(p1.y+p2.y)/2;
	GraphR->DrawText(p1,"SATELLITE NO",CColor[2],0,0,90);
	
	if (!BtnSol1->Down) return;
	
	if (level&&PlotStyle<=2) {
		GraphR->GetScale(xs,ys);
		for (int i=0;i<Obs.n;i++) {
			obs=&Obs.data[i];
			if ((color=ObsColor(obs,El[i]))<0) continue;
			
			xt=TimePos(obs->time);
			if (fabs(xt-tt[obs->sat-1])/xs>=1.0) {
				GraphR->DrawMark(xt,yp[obs->sat-1],0,PlotStyle<2?MColor[color]:CColor[3],
								 PlotStyle<2?MarkSize:0,0);
				tt[obs->sat-1]=xt;
			}
		}
	}
	if (level&&PlotStyle<=2) {
		DrawSlip(yp);
		DrawEphs(yp);
	}
	if (BtnShowTrack->Down&&0<=ind&&ind<NObs) {
		int i=IndexObs[ind];
		time=Obs.data[i].time;
		
		GraphR->GetLim(xl,yl);
		xl[0]=xl[1]=TimePos(Obs.data[i].time);
		GraphR->DrawPoly(xl,yl,2,CColor[2],0);
		
		for (;i<Obs.n&&timediff(Obs.data[i].time,time)==0.0;i++) {
			obs=&Obs.data[i];
			if ((color=ObsColor(obs,El[i]))<0) continue;
			GraphR->DrawMark(xl[0],yp[obs->sat-1],0,MColor[color],MarkSize*2+2,0);
		}
		GraphR->DrawMark(xl[0],yl[1]-1E-6,0,CColor[2],3,0);
		GraphR->DrawMark(xl[0],yl[1]-1E-6,1,CColor[2],9,0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawSlip(double *yp)
{
	obsd_t *obs;
	TPoint ps[2];
	double gfp[MAXSAT]={0},gf;
	int slip[2];
	
	if (ShowHalfC) {
		for (int i=0;i<Obs.n;i++) {
			obs=&Obs.data[i];
			
			if (El[i]<ElMask*D2R) continue;
			if (!((ObsType->ItemIndex==0||ObsType->ItemIndex==1)&&(obs->LLI[0]&2))&&
			    !((ObsType->ItemIndex==0||ObsType->ItemIndex==2)&&(obs->LLI[1]&2))) continue;
			
			if (GraphR->ToPoint(TimePos(obs->time),yp[obs->sat-1],ps[0])) {
				ps[1].x=ps[0].x;
				ps[1].y=ps[0].y+MarkSize*3/2+1;
				ps[0].y=ps[0].y-MarkSize*3/2;
				GraphR->DrawPoly(ps,2,MColor[0],0);
			}
		}
	}
	if (ShowSlip) {
		for (int i=0;i<Obs.n;i++) {
			obs=&Obs.data[i];
			slip[0]=obs->LLI[0]&1;
			slip[1]=obs->LLI[1]&1;
			slip[2]=0;
			if (obs->L[0]!=0.0&&obs->L[1]!=0.0) {
				gf=CLIGHT*(obs->L[0]/FREQ1-obs->L[1]/FREQ2);
				if (fabs(gfp[obs->sat-1]-gf)>THRESLIP) slip[2]=1;
				gfp[obs->sat-1]=gf;
			}
			if (El[i]<ElMask*D2R) continue;
			
			if (!(ShowSlip==2&&(ObsType->ItemIndex==0||ObsType->ItemIndex==1)&&slip[0])&&
			    !(ShowSlip==2&&(ObsType->ItemIndex==0||ObsType->ItemIndex==2)&&slip[1])&&
			    !(ShowSlip==1&&ObsType->ItemIndex<=2&&slip[2])) continue;
			
			if (GraphR->ToPoint(TimePos(obs->time),yp[obs->sat-1],ps[0])) {
				ps[1].x=ps[0].x;
				ps[1].y=ps[0].y+MarkSize*3/2+1;
				ps[0].y=ps[0].y-MarkSize*3/2;
				GraphR->DrawPoly(ps,2,MColor[5],0);
			}
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawEphs(double *yp)
{
	if (!ShowEph) return;
	
	for (int i=0;i<MAXSAT;i++) {
		int off[MAXSAT]={0};
		
		for (int j=0;j<Nav.n;j++) {
			if (Nav.eph[j].sat!=i+1) continue;
			TPoint ps[3];
			GraphR->ToPoint(TimePos(Nav.eph[j].ttr),yp[i],ps[0]);
			int in=GraphR->ToPoint(TimePos(Nav.eph[j].toe),yp[i],ps[2]);
			ps[1]=ps[0];
			off[Nav.eph[j].sat-1]=off[Nav.eph[j].sat-1]?0:3;
			
			for (int k=0;k<3;k++) ps[k].y+=MarkSize+2+off[Nav.eph[j].sat-1];
			ps[0].y-=2;
			
			GraphR->DrawPoly(ps,3,Nav.eph[j].svh?MColor[5]:CColor[1],0);
			
			if (in) GraphR->DrawMark(ps[2],0,Nav.eph[j].svh?MColor[5]:CColor[1],
									 Nav.eph[j].svh?4:3,0);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawSats(int level)
{
	TPoint p1,p2;
	AnsiString s,ss;
	obsd_t *obs;
	double x,y,xp,yp,xs,ys,dx,dy,xl[2],yl[2],r,p0[MAXSAT][2]={{0}};
	double p[MAXSAT][2]={{0}},gfp[MAXSAT]={0},gf;
	int ind=TimeScroll->Position,type=ObsType->ItemIndex;
	int hh=(int)(Disp->Font->Size*1.5),prn,color,slip[3];
	char id[16];
	
	GraphS->GetLim(xl,yl);
	r=(xl[1]-xl[0]<yl[1]-yl[0]?xl[1]-xl[0]:yl[1]-yl[0])*0.45;
	GraphS->DrawSkyPlot(0.0,0.0,CColor[1],CColor[2],r*2.0);
	
	if (!BtnSol1->Down) return;
	
	GraphS->GetScale(xs,ys);
	
	if (PlotStyle<=2) {
		for (int i=0;i<Obs.n;i++) {
			obs=&Obs.data[i];
			if (!(satsys(obs->sat,&prn)&NavSys)||El[i]<=0.0) continue;
			if ((color=ObsColor(obs,El[i]))<0) continue;
			
			x =r*sin(Az[i])*(1.0-2.0*El[i]/PI);
			y =r*cos(Az[i])*(1.0-2.0*El[i]/PI);
			xp=p[obs->sat-1][0];
			yp=p[obs->sat-1][1];
			
			if ((x-xp)*(x-xp)+(y-yp)*(y-yp)>=xs*xs) {
				GraphS->DrawMark(x,y,0,PlotStyle<2?MColor[color]:CColor[3],
								 PlotStyle<2?MarkSize:1,0);
				p[obs->sat-1][0]=x;
				p[obs->sat-1][1]=y;
			}
			if (xp==0.0&&yp==0.0) {
				p0[obs->sat-1][0]=x;
				p0[obs->sat-1][1]=y;
			}
		}
	}
	if (PlotStyle==0||PlotStyle==2) {
		for (int i=0;i<MAXSAT;i++) {
			if (p0[i][0]!=0.0||p0[i][1]!=0.0) {
				satno2id(i+1,id); s=id;
				GraphS->DrawText(p0[i][0],p0[i][1],s,CColor[2],1,0,0);
			}
		}
	}
	if (!level) return;
	
	if (ShowSlip&&PlotStyle<=2) {
		for (int i=0;i<Obs.n;i++) {
			obs=&Obs.data[i];
			if (!(satsys(obs->sat,&prn)&NavSys)||El[i]<=0.0) continue;
			
			slip[0]=obs->LLI[0];
			slip[1]=obs->LLI[1];
			slip[2]=0;
			if (obs->L[0]!=0.0&&obs->L[1]!=0.0) {
				gf=CLIGHT*(obs->L[0]/FREQ1-obs->L[1]/FREQ2);
				if (fabs(gfp[obs->sat-1]-gf)>THRESLIP) slip[2]=1;
				gfp[obs->sat-1]=gf;
			}
			x=r*sin(Az[i])*(1.0-2.0*El[i]/PI);
			y=r*cos(Az[i])*(1.0-2.0*El[i]/PI);
			
			if (p[obs->sat-1][0]==0.0&&p[obs->sat-1][1]==0.0) {
				p[obs->sat-1][0]=x; p[obs->sat-1][1]=y;
				continue;
			}
			dx=x-p[obs->sat-1][0];
			dy=y-p[obs->sat-1][1];
			p[obs->sat-1][0]=x;
			p[obs->sat-1][1]=y;
			
			if (El[i]<ElMask*D2R||(dx==0.0&&dy==0.0)) continue;
			
			if ((ShowSlip==2&&(type==0||type==1)&&slip[0])||
			    (ShowSlip==2&&(type==0||type==2)&&slip[1])||
			    (ShowSlip==1&&type<=2&&slip[2])) {
				GraphS->DrawMark(x,y,4,MColor[5],MarkSize*3+2,atan2(dy,dx)*R2D+90);
			}
		}
	}
	if (BtnShowTrack->Down&&0<=ind&&ind<NObs) {
		
		for (int i=IndexObs[ind];i<Obs.n&&i<IndexObs[ind+1];i++) {
			obs=&Obs.data[i];
			if (!(satsys(obs->sat,&prn)&NavSys)||El[i]<=0.0) continue;
			if ((color=ObsColor(obs,El[i]))<0) continue;
			
			x=r*sin(Az[i])*(1.0-2.0*El[i]/PI);
			y=r*cos(Az[i])*(1.0-2.0*El[i]/PI);
			
			satno2id(obs->sat,id);
			GraphS->DrawMark(x,y,0,MColor[color],Disp->Font->Size*2+5,0);
			GraphS->DrawMark(x,y,1,color==0?MColor[0]:CColor[2],Disp->Font->Size*2+5,0);
			GraphS->DrawText(x,y,s=id,CColor[0],0,0,0);
		}
	}
	if (ShowStats&&BtnShowTrack->Down) { // statistics
		
		GraphS->GetPos(p1,p2);
		p1.x=p2.x-10;
		p1.y+=8;
		s.sprintf("SAT : %-*s %-*s %-*s",NFREQ*2,"OBS",NFREQ*3,"SNR",NFREQ,"LLI");
		GraphS->DrawText(p1,s,CColor[2],2,2,0);
		p1.y+=3;
		
		for (int i=IndexObs[ind];i<Obs.n&&i<IndexObs[ind+1];i++) {
			obs=&Obs.data[i];
			if (!(satsys(obs->sat,&prn)&NavSys)||El[i]<=0.0) continue;
			if (HideLowSat&&El[i]<ElMask*D2R) continue;
			
			satno2id(obs->sat,id);
			s.sprintf("%-3s : ",id);
			for (int j=0;j<NFREQ;j++) s+=obs->L[j]==0.0?"_":"L";
			for (int j=0;j<NFREQ;j++) s+=obs->P[j]==0.0?"_":"P";
			s+=" ";
			for (int j=0;j<NFREQ;j++) s+=ss.sprintf("%02d ",obs->SNR[j]);
			for (int j=0;j<NFREQ;j++) s+=ss.sprintf("%d"   ,obs->LLI[j]);
			
			color=ObsColor(obs,El[i]);
			p1.y+=hh;
			GraphS->DrawText(p1,s,MColor[color<0?0:color],2,2,0);
		}
	}
	if (Nav.n<=0) {
		GraphS->GetPos(p1,p2);
		p2.x-=10;
		p2.y-=3;
		GraphS->DrawText(p2,"No Navigation Data",CColor[2],2,1,0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawDop(int level)
{
	AnsiString label;
	TPoint p1,p2;
	gtime_t time;
	double xp,xc,yc,xl[2],yl[2],azel[MAXSAT*2],*dop,*x,*y;
	int *ns,prn,n=0;
	
	GraphR->XLPos=TimeLabel?6:1; 
	GraphR->YLPos=1;
	GraphR->Week=Week;
	GraphR->GetLim(xl,yl);
	yl[0]=0.0; yl[1]=DOPLIM;
	GraphR->SetLim(xl,yl);
	GraphR->SetTick(0.0,0.0);
	int ind=TimeScroll->Position,doptype=DopType->ItemIndex;
	
	if (0<=ind&&ind<NObs&&BtnShowTrack->Down&&BtnCenterTrack->Down) {
		xp=TimePos(Obs.data[IndexObs[ind]].time);
		GraphR->GetCent(xc,yc);
		GraphR->SetCent(xp,yc);
	}
	GraphR->DrawAxis(1,1);
	GraphR->GetPos(p1,p2);
	p1.x=Disp->Font->Size;
	p1.y=(p1.y+p2.y)/2;
	label.sprintf("# OF SATELLITES / DOP (EL>=%.0f%s)",ElMask,CHARDEG);
	GraphR->DrawText(p1,label,CColor[2],0,0,90);
	
	if (!BtnSol1->Down) return;
	
	x  =new double[NObs];
	y  =new double[NObs];
	dop=new double[NObs*4];
	ns =new int   [NObs];
	
	for (int i=0;i<NObs;i++) {
		ns[n]=0;
		for (int j=IndexObs[i];j<Obs.n&&j<IndexObs[i+1];j++) {
			if (!(satsys(Obs.data[j].sat,&prn)&NavSys)) continue;
			if (El[j]<ElMask*D2R) continue;
			azel[  ns[n]*2]=Az[j];
			azel[1+ns[n]*2]=El[j];
			ns[n]++;
		}
		dops(ns[n],azel,ElMask*D2R,dop+n*4);
		x[n++]=TimePos(Obs.data[IndexObs[i]].time);
	}
	for (int i=0;i<4;i++) {
		if (doptype!=0&&doptype!=i+2) continue;
		
		for (int j=0;j<n;j++) y[j]=dop[i+j*4];
		
		if (!(PlotStyle%2)) {
			GraphR->DrawPoly(x,y,n,CColor[3],0);
		}
		if (level&&PlotStyle<2) {
			for (int j=0;j<n;j++) {
				GraphR->DrawMark(x[j],y[j],0,MColor[i+2],MarkSize,0);
			}
		}
	}
	if (doptype==0||doptype==1) {
		for (int i=0;i<n;i++) y[i]=ns[i];
		
		if (!(PlotStyle%2)) {
			GraphR->DrawPoly(x,y,n,CColor[3],1);
		}
		if (level&&PlotStyle<2) {
			for (int i=0;i<n;i++) {
				GraphR->DrawMark(x[i],y[i],0,MColor[1],MarkSize,0);
			}
		}
	}
	if (ShowStats) DrawStatDop(dop,ns,n);
	
	if (BtnShowTrack->Down&&0<=ind&&ind<NObs) {
		GraphR->GetLim(xl,yl);
		xl[0]=xl[1]=TimePos(Obs.data[IndexObs[ind]].time);
		
		GraphR->DrawPoly(xl,yl,2,CColor[2],0);
		
		ns[0]=0;
		for (int i=IndexObs[ind];i<Obs.n&&i<IndexObs[ind+1];i++) {
			if (!(satsys(Obs.data[i].sat,&prn)&NavSys)) continue;
			if (El[i]<ElMask*D2R) continue;
			azel[  ns[0]*2]=Az[i];
			azel[1+ns[0]*2]=El[i];
			ns[0]++;
		}
		dops(ns[0],azel,ElMask*D2R,dop);
		
		for (int i=0;i<4;i++) {
			if (doptype!=0&&doptype!=i+2) continue;
			GraphR->DrawMark(xl[0],dop[i],0,MColor[i+2],MarkSize*2+2,0);
		}
		if (doptype==0||doptype==1) {
			GraphR->DrawMark(xl[0],ns[0],0,MColor[1],MarkSize*2+2,0);
		}
		GraphR->DrawMark(xl[0],yl[1]-1E-6,0,CColor[2],3,0);
		GraphR->DrawMark(xl[0],yl[1]-1E-6,1,CColor[2],9,0);
	}
	if (Nav.n<=0&&(doptype==0||doptype>=2)) {
		GraphR->GetPos(p1,p2);
		p2.x-=10;
		p2.y-=3;
		GraphS->DrawText(p2,"No Navigation Data",CColor[2],2,1,0);
	}
	delete [] x;
	delete [] y;
	delete [] dop;
	delete [] ns;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawStatDop(double *dop, int *ns, int n)
{
	AnsiString s0[MAXOBS+1],s1[MAXOBS+1],s2[MAXOBS+1];
	TPoint p1,p2,p3,p4;
	double ave[4]={0};
	int m=0,ndop=0,nsat[MAXOBS]={0},fonth=(int)(Disp->Font->Size*1.5);
	
	for (int i=0;i<n;i++) {
		nsat[ns[i]]++;
		if (dop[i*4]<=0.0) continue;
		for (int j=0;j<4;j++) ave[j]+=dop[j+i*4];
		ndop++;
	}
	if (ndop>0) {
		for (int i=0;i<4;i++) ave[i]/=ndop;
	}
	if (DopType->ItemIndex==0||DopType->ItemIndex>=2) {
		s2[m++].sprintf("AVE= GDOP:%4.1f PDOP:%4.1f HDOP:%4.1f VDOP:%4.1f",
					    ave[0],ave[1],ave[2],ave[3]);
	}
	if (DopType->ItemIndex<=1) {
		
		for (int i=0,j=0;i<MAXOBS;i++) {
			if (nsat[i]<=0) continue;
			s0[m].sprintf("%s%2d:",j++==0?"NSAT= ":"",i);
			s1[m].sprintf("%7d",nsat[i]);
			s2[m++].sprintf("(%4.1f%%)",nsat[i]*100.0/n);
		}
	}
	GraphR->GetPos(p1,p2);
	p1.x=p2.x-10;
	p1.y+=8;
	p2=p1; p2.x-=fonth*4;
	p3=p2; p3.x-=fonth*8;
	
	for (int i=0;i<m;i++,p1.y+=fonth,p2.y+=fonth,p3.y+=fonth) {
		GraphR->DrawText(p3,s0[i],CColor[2],2,2,0);
		GraphR->DrawText(p2,s1[i],CColor[2],2,2,0);
		GraphR->DrawText(p1,s2[i],CColor[2],2,2,0);
	}
}
//---------------------------------------------------------------------------
int __fastcall TPlot::SearchPos(int x, int y)
{
	int sel=!BtnSol1->Down&&BtnSol2->Down?1:0;
	if (!BtnShowTrack->Down||(!BtnSol1->Down&&!BtnSol2->Down)) return -1;
	
	double xp,yp,xs,ys,xi,yi,zi,r;
	TPoint p(x,y);
	
	GraphT->ToPos(p,xp,yp);
	GraphT->GetScale(xs,ys);
	
	r=(MarkSize/2+2)*xs;
	
	for (int i=0;i<Sol[sel].n;i++) {
		if (QFlag->ItemIndex>0&&Sol[sel].data[i].posq!=QFlag->ItemIndex) continue;
		
		SolToXyz(Sol[sel].data+i,1,NULL,&xi,&yi,&zi,NULL,NULL,NULL,NULL,NULL,0,0);
		if ((xp-xi)*(xp-xi)+(yp-yi)*(yp-yi)<=r*r) return i;
	}
	return -1;
}
//---------------------------------------------------------------------------
int __fastcall TPlot::ObsColor(const obsd_t *obs, double el)
{
	double snr,sns[]={45.0,40.0,35.0,30.0,25.0};
	int color=1,type=ObsType->ItemIndex,freq;
	
	if (HideLowSat&&el<ElMask*D2R) return -1;
	
	if (type==0) {
		if		(obs->L[0]!=0.0&&obs->L[1]!=0.0) color=1;
		else if (obs->L[0]!=0.0) color=2;
		else if (obs->L[1]!=0.0) color=3;
		else if (obs->P[0]!=0.0&&obs->P[1]) color=4;
		else if (obs->P[0]!=0.0) color=5;
		else if (obs->P[1]!=0.0) color=6;
	}
	else {
		freq=(type-1)%NFREQ;
		if (type-1< NFREQ&&obs->L[freq]==0.0) return -1;
		if (type-1>=NFREQ&&obs->P[freq]==0.0) return -1;
		snr=obs->SNR[freq];
		for (color=1;color<6;color++) if (snr>sns[color-1]) break;
	}
	return el<ElMask*D2R?0:color;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateColor(void)
{
	for (int i=0;i<3;i++) {
		GraphT->Color[i]=CColor[i];
		GraphR->Color[i]=CColor[i];
		GraphS->Color[i]=CColor[i];
		
		for (int j=0;j<3;j++) {
			GraphG[j]->Color[i]=CColor[i];
		}
	}
	Disp->Font->Assign(Font);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateSize(void)
{
	int w=Disp->Width,h=Disp->Height;
	int tm=5,bm=(int)(Disp->Font->Size*1.5)+3,rm=5,lm=Disp->Font->Size*3+15;
	
	TPoint p1(0,0),p2(w,h);
	GraphT->SetPos(p1,p2);
	GraphS->SetPos(p1,p2);
	
	double xs,ys;
	GraphS->GetScale(xs,ys);
	xs=MAX(xs,ys);
	GraphS->SetScale(xs,xs);
	p1.x+=lm; p1.y+=tm; p2.x-=rm; p2.y=p2.y-bm;
	GraphR->SetPos(p1,p2);
	p2.y=p1.y;
	
	TSpeedButton *btn[]={BtnOn1,BtnOn2,BtnOn3};
	
	int n=0;
	for (int i=0;i<3;i++) if (btn[i]->Down) n++;
	for (int i=0;i<3;i++) {
		if (!btn[i]->Down) continue;
		int hh=(h-tm-bm)/n;
		p2.y+=hh; GraphG[i]->SetPos(p1,p2); p1.y+=hh;
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateSelect(void)
{
	UpdateEnable();
	
	if (PlotType<=3) {
		int sel=BtnSol1->Down||!BtnSol2->Down?0:1;
		TimeScroll->Position=0;
		TimeScroll->Max=Sol[sel].n<=0?100:Sol[sel].n-1;
	}
	UpdateInfo();
	UpdateTime();
	Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateTime(void)
{
	AnsiString msg,msgs[6];
	double *pos,vel,dir,r,az,el,azel[MAXSAT*2],dop[4];
	int no=0,ns=0,prn;
	int ind=TimeScroll->Position,sel=BtnSol1->Down||!BtnSol2->Down?0:1;
	char s[256],*p=s;
	
	if (!BtnShowTrack->Down) return;
	
	if (PlotType<=3&&(BtnSol1->Down||BtnSol2->Down||BtnSol12->Down)) {
		
		if (Sol[sel].n<=0) return;
		ind=ind<Sol[sel].n?ind:Sol[sel].n-1;
		pos=Sol[sel].data[ind].pos;
		
		if (PosType==0) {
			p+=sprintf(p,"%12.8f" CHARDEG " %13.8f" CHARDEG " %8.3fm",
					   pos[0]*R2D,pos[1]*R2D,pos[2]);
			
			if (ind>0) {
				vel=SolToVel(Sol[sel].data[ind].time,Sol[sel].data[ind-1].time,
							 pos,Sol[sel].data[ind-1].pos,dir);
				sprintf(p," V=%4.1fm/s (%3.0f" CHARDEG ")",vel,dir);
			}
		}
		else {
			r=norm(pos,3);
			az=norm(pos,2)<=1E-12?0.0:atan2(pos[0],pos[1])*R2D;
			el=r<=1E-12?0.0:asin(pos[2]/r)*R2D;
			
			sprintf(p,"B=%.3fm P=%.3f %.3f %.3fm D=%6.2f" CHARDEG " %5.2f" CHARDEG,
				    r,pos[0],pos[1],pos[2],az<0.0?az+360.0:az,el);
		}
		msg.sprintf("[%d]%s : %s",sel+1,TimeStr(Sol[sel].data[ind].time,3,1).c_str(),s);
	}
	else if (PlotType<=6&&BtnSol1->Down) {
		
		for (int i=IndexObs[ind];i<Obs.n&&i<IndexObs[ind+1];i++,no++) {
			if (!(satsys(Obs.data[i].sat,&prn)&NavSys)||El[i]<ElMask*D2R) continue;
			azel[  ns*2]=Az[i];
			azel[1+ns*2]=El[i];
			ns++;
		}
		dops(ns,azel,ElMask*D2R,dop);
		msg.sprintf("[1]%s : N=%d %s",TimeStr(Obs.data[IndexObs[ind]].time,3,1).c_str(),
		            no,PlotType==6?"":msgs[0].c_str());
		
		msgs[0].sprintf("NSAT=%d",ns);
		
		if (PlotType==6) {
			msgs[1].sprintf(" GDOP=%.1f",dop[0]);
			msgs[2].sprintf(" PDOP=%.1f",dop[1]);
			msgs[3].sprintf(" HDOP=%.1f",dop[2]);
			msgs[4].sprintf(" VDOP=%.1f",dop[3]);
			ShowLegend(msgs);
		}
	}
	ShowMsg(msg);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateInfo(void)
{
	int sel=BtnSol1->Down||!BtnSol2->Down?0:1,n=0,ne=0,nq[8]={0};
	gtime_t ts={0},te={0},t;
	AnsiString str="",s1,s2,msgs[6];
	
	if (PlotType<=3&&(BtnSol1->Down||BtnSol2->Down)) {
		
		double r[3],rr[3],b,bl[2]={1E6,0.0};
		
		pos2ecef(RPos[sel][0],rr);
		
		for (int i=0;i<Sol[sel].n;i++) {
			
			t=Sol[sel].data[i].time;
			if (ts.time==0) ts=t; te=t;
			if (PosType==0) {
				pos2ecef(Sol[sel].data[i].pos,r);
				for (int j=0;j<3;j++) r[j]-=rr[j];
				b=norm(r,3);
			}
			else {
				b=norm(Sol[sel].data[i].pos,3);
			}
			if (b<bl[0]) bl[0]=b; if (b>bl[1]) bl[1]=b;
			n++; nq[Sol[sel].data[i].posq]++;
		}
		if (!BtnShowTrack->Down&&n>0) {
			
			s1=TimeStr(ts,0,0); s2=TimeStr(te,0,1);
			str.sprintf("[%d]%s-%s : N=%d",sel+1,s1.c_str(),s2.c_str()+(TimeLabel?5:0),n);
			
			if (bl[0]+100.0<bl[1])
				str+=s1.sprintf(" B=%.1f-%.1fkm",bl[0]/1E3,bl[1]/1E3);
			else
				str+=s1.sprintf(" B=%.1fkm",bl[0]/1E3);
			str+=" Q=";
			for (int i=1;i<=6;i++) {
				if (nq[i]<=0) continue;
				msgs[i-1].sprintf("%d:%d(%.1f%%) ",i,nq[i],(double)nq[i]/n*100.0);
			}
		}
	}
	else if (PlotType<=3&&BtnSol12->Down) {
		
		for (int i=0,j=0;i<Sol[0].n&&j<Sol[1].n;i++,j++) {
			t=Sol[0].data[i].time;
			double tt=timediff(t,Sol[1].data[j].time);
			if (tt<-TTOL) {j--; continue;} else if (tt>TTOL) {i--; continue;}
			if (ts.time==0) ts=t; te=t;
			int q1=Sol[0].data[i].posq,q2=Sol[1].data[j].posq,q;
			n++; nq[q1>q2?q1:q2]++;
		}
		if (!BtnShowTrack->Down&&n>0) {
			s1=TimeStr(ts,0,0); s2=TimeStr(te,0,1);
			str.sprintf("[1-2]%s-%s : N=%d Q=",s1.c_str(),s2.c_str()+(TimeLabel?5:0),n);
			for (int i=1;i<=6;i++) {
				if (nq[i]<=0) continue;
				msgs[i-1].sprintf("%d:%d(%.1f%%) ",i,nq[i],(double)nq[i]/n*100.0);
			}
		}
	}
	else if (PlotType<=6&&BtnSol1->Down) {
		
		gtime_t tp={0};
		
		for (int i=0;i<Obs.n;i++) {
			t=Obs.data[i].time;
			if (ts.time==0) ts=t; te=t;
			if (tp.time==0||timediff(t,tp)>TTOL) ne++; 
			n++; tp=t; 
		}
		if (n>0) {
			s1=TimeStr(ts,0,0); s2=TimeStr(te,0,1);
			str.sprintf("[%d]%s-%s : EP=%d N=%d",sel+1,s1.c_str(),
				s2.c_str()+(TimeLabel?5:0),ne,n);
		}
		if (PlotType==6) {
			AnsiString m[]={"  NSAT"," GDOP"," PDOP"," HDOP"," VDOP",""};
			for (int i=0;i<6;i++) msgs[i]=m[i];
		}
		else {
			int type=ObsType->ItemIndex;
			AnsiString m1[]={" OBS=L1/2 "," L1 "," L2 "," P1/2 "," P1 "," P2 "};
			AnsiString m2[]={" SNR=>45 "," >40 "," >35 "," >30 "," >25 "," <25 "};
			for (int i=0;i<6;i++) {
				if (type==0) msgs[i]=m1[i]; else msgs[i]=m2[i];
			}
		}
	}
	ShowMsg(str); ShowLegend(msgs);
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdatePoint(int x, int y)
{
	AnsiString msg;
	gtime_t time;
	TPoint p(x,y);
	double e[3]={0},rr[3],dr[3],pos[3],xx,yy,r,xl[2],yl[2],q[2],az,el;
	char s[64];
	
	if (PlotType==0) { // map plot
		if (OPos[0]!=0.0||OPos[1]!=0.0) {
			GraphT->ToPos(p,e[0],e[1]);
			pos2ecef(OPos,rr);
			enu2ecef(OPos,e,dr);
			for (int i=0;i<3;i++) rr[i]+=dr[i];
			ecef2pos(rr,pos);
			msg.sprintf("%12.8f" CHARDEG " %13.8f" CHARDEG,pos[0]*R2D,pos[1]*R2D);
		}
	}
	else if (PlotType<=4||PlotType==6) { // time-graph plot
		
		if (PlotType<=3) GraphG[0]->ToPos(p,xx,yy);
		else			 GraphR   ->ToPos(p,xx,yy);
		time=gpst2time(Week,xx);
		if      (TimeLabel==2) time=utc2gpst(time); // UTC
		else if (TimeLabel==3) time=timeadd(gpst2utc(time),-9*3600.0); // JST
		msg=TimeStr(time,0,1);
	}
	else if (PlotType==5) { // sky-plot
		
		GraphS->GetLim(xl,yl);
		GraphS->ToPos(p,q[0],q[1]);
		r=(xl[1]-xl[0]<yl[1]-yl[0]?xl[1]-xl[0]:yl[1]-yl[0])*0.45;
		
		if ((el=90.0-90.0*norm(q,2)/r)>0.0) {
			az=el>=90.0?0.0:atan2(q[0],q[1])*R2D;
			if (az<0.0) az+=360.0;
			msg.sprintf("AZ=%5.1f" CHARDEG " EL=%4.1f" CHARDEG,az,el);
		}
	}
	Panel22->Visible=true;
	Message2->Caption=msg;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateOrigin(void)
{
	double opos[3]={0},rr[3],r[3],e[3],x,y;
	
	if (Sol[0].n<=0) return;
	
	switch (Origin) {
		case 0: // start pos
			for (int i=0;i<3;i++) opos[i]=Sol[0].data[0].pos[i];
			break;
		case 1: // end pos
			for (int i=0;i<3;i++) opos[i]=Sol[0].data[Sol[0].n-1].pos[i];
			break;
		case 2: // average pos
			for (int i=0;i<3;i++) {
				for (int j=0;j<Sol[0].n;j++) opos[i]+=Sol[0].data[j].pos[i];
				opos[i]/=Sol[0].n;
			}
			break;
		case 3: // reference station pos
			for (int i=0;i<3;i++) opos[i]=RPos[0][0][i];
			break;
		case 4: // lat/lon/height
			if (PosType==0) {
				for (int i=0;i<3;i++) opos[i]=OOPos[i];
			}
			else {
				for (int i=0;i<3;i++) opos[i]=0.0;
			}
			break;
	}
	if (YRange<=0) {
		pos2ecef(OPos,rr);
		pos2ecef(opos,r);
		for (int i=0;i<3;i++) r[i]-=rr[i];
		ecef2enu(OPos,r,e);
		GraphT->GetCent(x,y);
		GraphT->SetCent(x-e[0],y-e[1]);
		
		for (int i=0;i<3;i++) {
			GraphG[i]->GetCent(x,y);
			GraphG[i]->SetCent(x,y-e[i]);
		}
	}
	for (int i=0;i<3;i++) OPos[i]=opos[i];
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateLimTime(int all)
{
	double tl[2]={86400.0*7,0.0},tp[2],xl[2]={0,1},yl[2],zl[2];
	
	if (all||(1<=PlotType&&PlotType<=3)) {
		
		GraphG[0]->GetLim(tp,xl);
		GraphG[1]->GetLim(tp,yl);
		GraphG[2]->GetLim(tp,zl);
		
		if ((BtnSol1->Down||BtnSol12->Down)&&Sol[0].n>0) {
			tl[0]=MIN(tl[0],TimePos(Sol[0].data[0].time));
			tl[1]=MAX(tl[1],TimePos(Sol[0].data[Sol[0].n-1].time));
		}
		if ((BtnSol2->Down||BtnSol12->Down)&&Sol[1].n>0) {
			tl[0]=MIN(tl[0],TimePos(Sol[1].data[0].time));
			tl[1]=MAX(tl[1],TimePos(Sol[1].data[Sol[1].n-1].time));
		}
		GraphG[0]->SetLim(tl,xl);
		GraphG[1]->SetLim(tl,yl);
		GraphG[2]->SetLim(tl,zl);
	}
	if (all||PlotType==4||PlotType==6) {
		
		if (Obs.n>0) {
			tl[0]=TimePos(Obs.data[0].time);
			tl[1]=TimePos(Obs.data[Obs.n-1].time);
		}
		GraphR->SetLim(tl,xl);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateLimFit(int all)
{
	gtime_t *t1,*t2;
	double *x1,*y1,*z1,*x2,*y2,*z2,tt;
	double tl[2],xl[]={1E8,-1E8},yl[2]={1E8,-1E8},zl[2]={1E8,-1E8};
	double xs,ys,xp[2];
	int *q1,*q2,n1=Sol[0].n,n2=Sol[1].n,m,m1,m2;
	
	if (PlotType>3) return;
	
	if (BtnSol1->Down&&n1>0) {
		t1=new gtime_t[n1];
		x1=new double [n1];
		y1=new double [n1];
		z1=new double [n1];
		q1=new int    [n1];
		
		m=SolToXyz(Sol[0].data,n1,t1,x1,y1,z1,q1,NULL,NULL,NULL,NULL,QFlag->ItemIndex,PlotType-1);
		
		for (int i=0;i<m;i++) {
			xl[0]=MIN(xl[0],x1[i]);
			yl[0]=MIN(yl[0],y1[i]);
			zl[0]=MIN(zl[0],z1[i]);
			xl[1]=MAX(xl[1],x1[i]);
			yl[1]=MAX(yl[1],y1[i]);
			zl[1]=MAX(zl[1],z1[i]);
		}
		delete [] t1;
		delete [] x1;
		delete [] y1;
		delete [] z1;
		delete [] q1;
	}
	if (BtnSol2->Down&&n2>0) {
		t2=new gtime_t[n2];
		x2=new double [n2];
		y2=new double [n2];
		z2=new double [n2];
		q2=new int    [n2];
		
		m=SolToXyz(Sol[1].data,n2,t2,x2,y2,z2,q2,NULL,NULL,NULL,NULL,QFlag->ItemIndex,PlotType-1);
		
		for (int i=0;i<m;i++) {
			xl[0]=MIN(xl[0],x2[i]);
			yl[0]=MIN(yl[0],y2[i]);
			zl[0]=MIN(zl[0],z2[i]);
			xl[1]=MAX(xl[1],x2[i]);
			yl[1]=MAX(yl[1],y2[i]);
			zl[1]=MAX(zl[1],z2[i]);
		}
		delete [] t2;
		delete [] x2;
		delete [] y2;
		delete [] z2;
		delete [] q2;
	}
	if (BtnSol12->Down&&n1>0&&n2>0) {
		t1=new gtime_t[n1];
		t2=new gtime_t[n2];
		x1=new double [n1];
		y1=new double [n1];
		z1=new double [n1];
		x2=new double [n2];
		y2=new double [n2];
		z2=new double [n2];
		q1=new int    [n1];
		q2=new int    [n2];
		
		m1=SolToXyz(Sol[0].data,n1,t1,x1,y1,z1,q1,NULL,NULL,NULL,NULL,QFlag->ItemIndex,PlotType-1);
		m2=SolToXyz(Sol[1].data,n2,t2,x2,y2,z2,q2,NULL,NULL,NULL,NULL,QFlag->ItemIndex,PlotType-1);
		m=0;
		
		for (int i=0,j=0;i<m1&&j<m2;i++,j++) {
			tt=timediff(t1[i],t2[j]);
			if (tt<-TTOL) {j--; continue;} else if (tt>TTOL) {i--; continue;}
			x1[m]=x1[i]-x2[j];
			y1[m]=y1[i]-y2[j];
			z1[m]=z1[i]-z2[j];
			m++;
		}
		for (int i=0;i<m;i++) {
			xl[0]=MIN(xl[0],x1[i]);
			yl[0]=MIN(yl[0],y1[i]);
			zl[0]=MIN(zl[0],z1[i]);
			xl[1]=MAX(xl[1],x1[i]);
			yl[1]=MAX(yl[1],y1[i]);
			zl[1]=MAX(zl[1],z1[i]);
		}
		delete [] t1;
		delete [] t2;
		delete [] x1;
		delete [] y1;
		delete [] z1;
		delete [] x2;
		delete [] y2;
		delete [] z2;
		delete [] q1;
		delete [] q2;
	}
	xl[0]-=0.05;
	xl[1]+=0.05;
	yl[0]-=0.05;
	yl[1]+=0.05;
	zl[0]-=0.05;
	zl[1]+=0.05;
	
	if (all||PlotType==0) {
		GraphT->SetLim(xl,yl);
		GraphT->GetScale(xs,ys);
		GraphT->SetScale(MAX(xs,ys),MAX(xs,ys));
	}
	if (all||(1<=PlotType&&PlotType<=3)) {
		GraphG[0]->GetLim(tl,xp);
		GraphG[0]->SetLim(tl,xl);
		GraphG[1]->SetLim(tl,yl);
		GraphG[2]->SetLim(tl,zl);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateLimOri(int all)
{
	double xl[]={-YRange,YRange};
	double yl[]={-YRange,YRange};
	double zl[]={-YRange,YRange};
	double xs,ys,tl[2],xp[2];
	
	if (all||PlotType==0) {
		GraphT->SetLim(xl,yl);
		GraphT->GetScale(xs,ys);
		GraphT->SetScale(MAX(xs,ys),MAX(xs,ys));
	}
	if (all||(1<=PlotType&&PlotType<=3)) {
		GraphG[0]->GetLim(tl,xp);
		GraphG[0]->SetLim(tl,xl);
		GraphG[1]->SetLim(tl,yl);
		GraphG[2]->SetLim(tl,zl);
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::UpdateEnable(void)
{
	Panel1         ->Visible=MenuToolBar  ->Checked;
	Panel2         ->Visible=MenuStatusBar->Checked;
	
	PlotType=RawObs?PlotTypeObs->ItemIndex+4:PlotTypeSol->ItemIndex;
	PlotTypeSol    ->Visible=!RawObs;
	PlotTypeObs    ->Visible=RawObs;
	
	QFlag          ->Visible=!RawObs;
	ObsType        ->Visible=RawObs&&PlotTypeObs->ItemIndex<=1;
	DopType        ->Visible=RawObs&&PlotTypeObs->ItemIndex==2;
	
	BtnSol2        ->Enabled=PlotType<=3;
	BtnSol12       ->Enabled=PlotType<=3&&Sol[0].n>0&&Sol[1].n>0;
	BtnCenterOri   ->Enabled=PlotType<=3;
	BtnCenterTrack ->Enabled=(PlotType<=4||PlotType==6)&&BtnShowTrack->Down;
	BtnFitX        ->Enabled=(0<=PlotType&&PlotType<=4)||PlotType==6;
	BtnFitY        ->Enabled=PlotType<=3;
	BtnOn1         ->Enabled=1<=PlotType&&PlotType<=4;
	BtnOn2         ->Enabled=1<=PlotType&&PlotType<=4;
	BtnOn3         ->Enabled=1<=PlotType&&PlotType<=4;
	
	TimeScroll     ->Enabled=BtnShowTrack->Down;
	BtnAnimate     ->Enabled=BtnShowTrack->Down;
	MenuCenterOri  ->Enabled=BtnCenterOri->Enabled;
	MenuFitX       ->Enabled=BtnFitX->Enabled;
	MenuFitY       ->Enabled=BtnFitY->Enabled;
	MenuCenterTrack->Enabled=BtnCenterTrack->Enabled;
	MenuShowTrack  ->Checked=BtnShowTrack->Down;
	MenuCenterTrack->Checked=BtnCenterTrack->Down;
	MenuAnimStart  ->Enabled=BtnShowTrack->Down&&!BtnAnimate->Down;
	MenuAnimStop   ->Enabled=BtnShowTrack->Down&&BtnAnimate->Down;
}
//---------------------------------------------------------------------------
int __fastcall TPlot::SolToXyz(const solpd_t *data, int n, gtime_t *t,
	double *x, double *y, double *z, int *q, double *xs, double *ys,
	double *zs, double *xys, int qflag, int type)
{
	int m=0;
	double rr[3],r[3],e[3];
	
	pos2ecef(OPos,rr);
	
	for (int i=0;i<n;i++) {
		if (t) t[m]=data[i].time;
		if (PosType==0) {
			pos2ecef(data[i].pos,r);
			for (int j=0;j<3;j++) r[j]-=rr[j];
			ecef2enu(OPos,r,e);
			x[m]=e[0];
			y[m]=e[1];
			z[m]=e[2];
		}
		else {
			x[m]=data[i].pos[0]-OPos[0];
			y[m]=data[i].pos[1]-OPos[1];
			z[m]=data[i].pos[2]-OPos[2];
		}
		if (q  ) q  [m]=data[i].posq;
		if (xs ) xs [m]=data[i].poss[0];
		if (ys ) ys [m]=data[i].poss[1];
		if (zs ) zs [m]=data[i].poss[2];
		if (xys) xys[m]=data[i].poss[3];
		if (!qflag||data[i].posq==qflag) m++;
	}
	if (type>=1) m=SolDiff(t,x,y,z,q,m); // velocity
	if (type>=2) m=SolDiff(t,x,y,z,q,m); // acceleration
	
	return m;
}
//---------------------------------------------------------------------------
int __fastcall TPlot::SolDiff(gtime_t *time, double *x, double *y,
	double *z, int *q, int n)
{
	double tt;
	int m=0;
	
	if (n<=1) return 0;
	
	for (int i=0;i<n-1;i++) {
		if ((tt=timediff(time[i+1],time[i]))==0.0) continue;
		time[m]=timeadd(time[i],tt/2.0);
		x[m]=(x[i+1]-x[i])/tt;
		y[m]=(y[i+1]-y[i])/tt;
		z[m]=(z[i+1]-z[i])/tt;
		q[m]=q[i]<q[i+1]?q[i]:q[i+1];
		if (fabs(tt)<=MAXTDIFF) m++;
	}
	return m;
}
//---------------------------------------------------------------------------
double __fastcall TPlot::SolToVel(gtime_t time1, gtime_t time0,
	const double *pos1, const double *pos0, double &dir)
{
	double d,v,r0[3],r1[3],e[3];
	dir=0.0;
	pos2ecef(pos0,r0);
	pos2ecef(pos1,r1);
	for (int i=0;i<3;i++) r1[i]-=r0[i];
	ecef2enu(pos0,r1,e);
	if ((d=norm(e,2))<1E-3) return 0.0;
	if ((v=d/timediff(time1,time0))<0.05) return 0.0;
	dir=atan2(e[0],e[1])*R2D;
	if (dir<0.0) dir+=360.0;
	return v;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::DrawArrow(const gtime_t *t, const double *x,
	const double *y, int n)
{
	TPoint p;
	double tt,d[2],dist,dt,vel;
	int off=8;
	
	for (int i=1;i<n-1;i++) {
		tt=time2gpst(t[i],NULL);
		d[0]=x[i+1]-x[i-1];
		d[1]=y[i+1]-y[i-1];
		dist=norm(d,2);
		dt=timediff(t[i+1],t[i-1]);
		vel=dt==0.0?0.0:dist/dt;
		
		if (vel<0.5||fmod(tt+0.005,INTARROW)>=0.01) continue;
		
		GraphT->ToPoint(x[i],y[i],p);
		p.x-=(int)(off*d[1]/dist);
		p.y-=(int)(off*d[0]/dist);
		GraphT->DrawMark(p,10,CColor[3],15,(int)(atan2(d[1],d[0])*R2D));
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::CalcStats(const double *x, int n,
	double ref, double &ave, double &std, double &rms)
{
	double sum=0.0,sumsq=0.0;
	
	if (n<=0) {
		ave=std=rms=0.0;
		return;
	}
	ave=std=rms=0.0;
	
	for (int i=0;i<n;i++) {
		sum  +=x[i];
		sumsq+=x[i]*x[i];
	}
	ave=sum/n;
	std=n>1?SQRT((sumsq-2.0*sum*ave+ave*ave*n)/(n-1)):0.0;
	rms=SQRT((sumsq-2.0*sum*ref+ref*ref*n)/n);
}
//---------------------------------------------------------------------------
double __fastcall TPlot::TimePos(gtime_t time)
{
	double tow;
	int week;
	if		(TimeLabel==2) time=gpst2utc(time);					  // UTC
	else if (TimeLabel==3) time=timeadd(gpst2utc(time),9*3600.0); // JST
	tow=time2gpst(time,&week);
	return tow+(week-Week)*86400.0*7;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TPlot::TimeStr(gtime_t time, int n, int tsys)
{
	AnsiString str;
	char s[64];
	int week;
	double tow;
	
	switch (TimeLabel) {
		case 0: // www/ssss
			tow=time2gpst(time,&week);
			sprintf(s,"%4d/%*.*fs",week,(n>0?6:5)+n,n,tow);
			break;
		case 1: // gpst
			time2str(time,s,n);
			break;
		case 2: // utc
			time2str(gpst2utc(time),s,n);
			break;
		case 3: // jst
			time2str(timeadd(gpst2utc(time),9*3600.0),s,n);
			break;
	}
	str=s;
	if (tsys&&TimeLabel) {
		str+=TimeLabel==2?" UTC":(TimeLabel==3?" JST":" GPST");
	}
	return str;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::ShowMsg(AnsiString msg)
{
	Message1->Caption=msg;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::ShowLegend(AnsiString *msgs)
{
	TLabel *ql[]={QL1,QL2,QL3,QL4,QL5,QL6};
	
	for (int i=0;i<6;i++) {
		if (!msgs||msgs[i]=="") {
			ql[i]->Caption=" "; ql[i]->Width=1;
		}
		else {
			ql[i]->Caption=msgs[i]; ql[i]->Font->Color=MColor[i+1];
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPlot::LoadOpt(void)
{
	TIniFile *ini=new TIniFile("rtkplot.ini");
    
	ObsFormat  =ini->ReadInteger("plot","obsformat",   0);
    
    TimeLabel  =ini->ReadInteger("plot","timelabel",   1);
    AutoScale  =ini->ReadInteger("plot","autoscale",   1);
    ShowStats  =ini->ReadInteger("plot","showstats",   0);
    ShowLabel  =ini->ReadInteger("plot","showlabel",   1);
    ShowGLabel =ini->ReadInteger("plot","showglabel",  0);
    ShowCompass=ini->ReadInteger("plot","showcompass", 0);
    ShowScale  =ini->ReadInteger("plot","showscale",   1);
    ShowArrow  =ini->ReadInteger("plot","showarrow",   0);
    ShowSlip   =ini->ReadInteger("plot","showslip",    0);
    ShowHalfC  =ini->ReadInteger("plot","showhalfc",   0);
    ShowErr    =ini->ReadInteger("plot","showerr",     0);
    ShowEph    =ini->ReadInteger("plot","showeph",     0);
    PlotStyle  =ini->ReadInteger("plot","plotstyle",   0);
    MarkSize   =ini->ReadInteger("plot","marksize",    3);
    NavSys     =ini->ReadInteger("plot","navsys",SYS_GPS);
    AnimCycle  =ini->ReadInteger("plot","animcycle",   1);
    HideLowSat =ini->ReadInteger("plot","hidelowsat",  0);
    
    MColor[0]=(TColor)ini->ReadInteger("plot","mcolor0",(int)clSilver);
    MColor[1]=(TColor)ini->ReadInteger("plot","mcolor1",(int)clGreen);
    MColor[2]=(TColor)ini->ReadInteger("plot","mcolor2",0x00AAFF);
    MColor[3]=(TColor)ini->ReadInteger("plot","mcolor3",(int)clFuchsia);
    MColor[4]=(TColor)ini->ReadInteger("plot","mcolor4",(int)clBlue);
    MColor[5]=(TColor)ini->ReadInteger("plot","mcolor5",(int)clRed);
    MColor[6]=clGray;
    CColor[0]=(TColor)ini->ReadInteger("plot","color1", (int)clWhite);
    CColor[1]=(TColor)ini->ReadInteger("plot","color2", (int)clSilver);
    CColor[2]=(TColor)ini->ReadInteger("plot","color3", (int)clBlack);
    CColor[3]=(TColor)ini->ReadInteger("plot","color4", (int)clSilver);
    
    RefDialog->StaPosFile=ini->ReadString("plot","staposfile","");
    
    ElMask    =ini->ReadFloat  ("plot","elmask", 0.0);
    YRange    =ini->ReadFloat  ("plot","yrange", 1.0);
    Origin    =ini->ReadInteger("plot","orgin",  0);
    OOPos[0]  =ini->ReadFloat  ("plot","oopos1", 0);
    OOPos[1]  =ini->ReadFloat  ("plot","oopos2", 0);
    OOPos[2]  =ini->ReadFloat  ("plot","oopos3", 0);
    
    Font->Charset=ANSI_CHARSET;
    Font->Name=ini->ReadString("plot","fontname","Tahoma");
    Font->Size=ini->ReadInteger("plot","fontsize",8);
    
	delete ini;
}
//---------------------------------------------------------------------------
void __fastcall TPlot::SaveOpt(void)
{
	TIniFile *ini=new TIniFile("rtkplot.ini");
	
    ini->WriteInteger("plot","obsformat",ObsFormat);
    
    ini->WriteInteger("plot","timelabel",  TimeLabel);
    ini->WriteInteger("plot","autoscale",  AutoScale);
    ini->WriteInteger("plot","showstats",  ShowStats);
    ini->WriteInteger("plot","showlabel",  ShowLabel);
    ini->WriteInteger("plot","showglabel", ShowGLabel);
    ini->WriteInteger("plot","showcompass",ShowCompass);
    ini->WriteInteger("plot","showscale",  ShowScale);
    ini->WriteInteger("plot","showarrow",  ShowArrow);
    ini->WriteInteger("plot","showslip",   ShowSlip);
    ini->WriteInteger("plot","showhalfc",  ShowHalfC);
    ini->WriteInteger("plot","showerr",    ShowErr);
    ini->WriteInteger("plot","showeph",    ShowEph);
    ini->WriteInteger("plot","plotstyle",  PlotStyle);
    ini->WriteInteger("plot","marksize",   MarkSize);
    ini->WriteInteger("plot","navsys",     NavSys);
    ini->WriteInteger("plot","animcycle",  AnimCycle);
    ini->WriteInteger("plot","hidelowsat", HideLowSat);
    
    ini->WriteInteger("plot","mcolor0",(int)MColor[0]);
    ini->WriteInteger("plot","mcolor1",(int)MColor[1]);
    ini->WriteInteger("plot","mcolor2",(int)MColor[2]);
    ini->WriteInteger("plot","mcolor3",(int)MColor[3]);
    ini->WriteInteger("plot","mcolor4",(int)MColor[4]);
    ini->WriteInteger("plot","mcolor5",(int)MColor[5]);
    ini->WriteInteger("plot","color1", (int)CColor[0]);
    ini->WriteInteger("plot","color2", (int)CColor[1]);
    ini->WriteInteger("plot","color3", (int)CColor[2]);
    ini->WriteInteger("plot","color4", (int)CColor[3]);
    
    ini->WriteString("plot","staposfile",RefDialog->StaPosFile);
    
    ini->WriteFloat  ("plot","elmask", ElMask);
    ini->WriteFloat  ("plot","yrange", YRange);
    ini->WriteInteger("plot","orgin",  Origin);
    ini->WriteFloat  ("plot","oopos1", OOPos[0]);
    ini->WriteFloat  ("plot","oopos2", OOPos[1]);
    ini->WriteFloat  ("plot","oopos3", OOPos[2]);
    
    ini->WriteString ("plot","fontname", Font->Name);
    ini->WriteInteger("plot","fontsize", Font->Size);
    
	delete ini;
}
