/*------------------------------------------------------------------------------
* str2str.c : divide a stream to multiple stream
*
*          Copyright (C) 2007-2009 by T.TAKASU, All rights reserved.
*
* version : $Revision: 1.1 $ $Date: 2008/07/17 21:54:53 $
* history : 2009/06/17  1.0 new
*-----------------------------------------------------------------------------*/
#include <signal.h>
#include <unistd.h>
#include "rtklib.h"

static const char rcsid[]="$Id:$";

static strsvr_t strsvr;        /* stream server */
static char startcmd[4096]="";
static char stopcmd[4096]="";
static int run=0;

/* help text -----------------------------------------------------------------*/
static const char *help[]={
"",
" usage: str2str [options]",
"",
" Input data from a stream and devide and output them to multiple streams",
" The input stream can be serial, tcp client, tcp server, ntrip client, or",
" file. The output stream can be serial, tcp client, tcp server, ntrip server,",
" or file. str2str is a resident type application. To stop it, type ctr-c in",
" console if run foreground or send signal SIGINT for background process.",
" Command options are as follows.",
"",
" -in str   input stream type/path",
"             serial://port[:brate[:bsize[:parity[:stopb[:fctr]]]]]",
"             (brate=bps,bsize=7|8,parity=n|o|e,stopb=1|2,fctr=off|xon|cts)",
"             tcpsvr://:port",
"             tcpcli://addr[:port]",
"             ntrip://[user[:passwd]@]addr[:port][/mntpnt]",
"             file://path",
" -out str  output stream type/path",
"             serial://port[:brate[:bsize[:parity[:stopb[:fctr]]]]]",
"             (brate=bps,bsize=7|8,parity=n|o|e,stopb=1|2,fctr=off|xon|cts)",
"             tcpsvr://:port",
"             tcpcli://addr[:port]",
"             ntrip://[user[:passwd]@]addr[:port][/mntpnt[:str]]",
"             file://path",
" -cs file  commands file at input startup [no]",
" -ce file  commands file at input shutdown [no]",
" -r lat lon nmea gpgga postion latitude/longitude (deg) [no]",
" -t file   trace file [no trace]",
" -l level  trace level [1]",
" -h        print help",
};
/* print help ----------------------------------------------------------------*/
static void printhelp(void)
{
    int i;
    for (i=0;i<sizeof(help)/sizeof(*help);i++) fprintf(stderr,"%s\n",help[i]);
    exit(0);
}
/* server stop ---------------------------------------------------------------*/
void stopsvr(int arg)
{
    strsvrstop(&strsvr,stopcmd);
    fprintf(stderr,"\n[str2str] stream server stop\n");
    run=0;
}
/* str2str -------------------------------------------------------------------*/
int main(int argc, char **argv)
{
    FILE *fp;
    unsigned int tick=0;
    int i,opts[]={10000,10000,2000,32768,10,0},strs[17]={0},nout=0,trlevel=1;
    int stat[17]={0},byte[17]={0},bps[17]={0},n;
    char *p,*paths[17],str[17][MAXSTRPATH]={{0}};
    char *trfile="",msg[MAXSTRMSG]="",*files[2]={0};
    double nmeapos[3]={0},pos[3]={0};
    
    for (i=0;i<17;i++) paths[i]=str[i];
    
    for (i=1;i<argc;i++) {
        if (!strcmp(argv[i],"-in")&&i+1<argc) {
            if (!(p=strstr(argv[++i],"://"))) {
                fprintf(stderr,"input steam path error : %s\n",argv[i]);
                return -1;
            }
            if (!strncmp(argv[i],"serial",6)) {
                strs[0]=STR_SERIAL; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"tcpsvr",6)) {
                strs[0]=STR_TCPSVR; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"tcpcli",6)) {
                strs[0]=STR_TCPCLI; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"ntrip",5)) {
                strs[0]=STR_NTRIPCLI; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"file",4)) {
                strs[0]=STR_FILE; strcpy(paths[0],p+3);
            }
            else {
                fprintf(stderr,"input stream path error : %s\n",argv[i]);
                return -1;
            }
        }
        else if (!strcmp(argv[i],"-out")&&i+1<argc&&nout<16) {
            if (!(p=strstr(argv[++i],"://"))) {
                fprintf(stderr,"output steam path error : %s\n",argv[i]);
                return -1;
            }
            if (!strncmp(argv[i],"serial",6)) {
                strs[1+nout]=STR_SERIAL; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"tcpsvr",6)) {
                strs[1+nout]=STR_TCPSVR; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"tcpcli",6)) {
                strs[1+nout]=STR_TCPCLI; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"ntrip",5)) {
                strs[1+nout]=STR_NTRIPSVR; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"file",4)) {
                strs[1+nout]=STR_FILE; strcpy(paths[1+nout++],p+3);
            }
            else {
                fprintf(stderr,"input stream path error : %s\n",argv[i]);
                return -1;
            }
        }
        else if (!strcmp(argv[i],"-r")&&i+2<argc) {
            pos[0]=atof(argv[++i])*D2R;
            pos[1]=atof(argv[++i])*D2R;
            pos2ecef(pos,nmeapos);
            opts[5]=5000;
        }
        else if (!strcmp(argv[i],"-cs")&&i+1<argc) files[0]=argv[++i];
        else if (!strcmp(argv[i],"-ce")&&i+1<argc) files[1]=argv[++i];
        else if (!strcmp(argv[i],"-t" )&&i+1<argc) trfile=argv[++i];
        else if (!strcmp(argv[i],"-l" )&&i+1<argc) trlevel=atoi(argv[++i]);
        else if (*argv[i]=='-') printhelp();
    }
    if (strs[0]==0||nout==0) {
        fprintf(stderr,"specify input stream and output stream(s)\n");
        return -1;
    }
    if (*trfile) {
        traceopen(trfile); tracelevel(trlevel);
    }
    if (files[0]) {
        if (!(fp=fopen(files[0],"rb"))) {
            fprintf(stderr,"command file open error: %s\n",files[0]);
            return -1;
        }
        n=fread(startcmd,1,sizeof(startcmd)-1,fp); startcmd[n]='\0';
        fclose(fp);
    }
    if (files[1]) {
        if (!(fp=fopen(files[1],"rb"))) {
            fprintf(stderr,"command file open error: %s\n",files[1]);
            return -1;
        }
        n=fread(stopcmd,1,sizeof(stopcmd)-1,fp); stopcmd[n]='\0';
        fclose(fp);
    }
    signal(SIGINT,stopsvr);
    signal(SIGPIPE,SIG_IGN); /* prevent termination by socket break */
    
    strsvrinit(&strsvr,nout);
    
    tick=tickget();
    
    fprintf(stderr,"[str2str] stream server start\n");
    strsvrstart(&strsvr,opts,strs,paths,startcmd,nmeapos);
    run=1;
    while (run) { /* sleep until signal catched */
        strsvrstat(&strsvr,stat,byte,bps,msg);
        fprintf(stderr,"[str2str] %6.1f: ",(int)(tickget()-tick)/1000.0);
        for (i=0;i<nout+1;i++) {
            fprintf(stderr,"(%d) %d %8d %6d ",i,stat[i],byte[i],bps[i]);
        }
        fprintf(stderr,": %-32s\r",msg);
        sleepms(100);
    }
    return 0;
}
