/*------------------------------------------------------------------------------
* rtklib unit test driver : glonass ephemeris function
*-----------------------------------------------------------------------------*/
#include <stdio.h>
#include <assert.h>
#include "../../src/rtklib.h"

static void dumpgeph(geph_t *geph, int n)
{
	char s1[64],s2[64];
	int i;
	for (i=0;i<n;i++) {
		time2str(geph[i].toe,s1,0);
		time2str(geph[i].tof,s2,0);
		printf("(%3d) sat=%2d frq=%2d svh=%d age=%d toe=%s %s "
			   "pos=%13.3f %13.3f %13.3f vel=%9.3f %9.3f %9.3f "
			   "acc=%10.3E %10.3E %10.3E taun=%12.5E gamn=%12.5E\n",
			i+1,geph[i].sat,geph[i].frq,geph[i].svh,geph[i].age,s1,s2,
			geph[i].pos[0],geph[i].pos[1],geph[i].pos[2],
			geph[i].vel[0],geph[i].vel[1],geph[i].vel[2],
			geph[i].acc[0],geph[i].acc[1],geph[i].acc[2],
			geph[i].taun,geph[i].gamn);
	}
}
/* readrnx() */
void utest1(void)
{
	char *file1[]={"brdd0910.09g"};
	char *file2[]={"brdc0910.09g"};
	obs_t obs={0};
	nav_t nav={0};
	int n;
	
	n=readrnx(file1,1,&obs,&nav);
		assert(n==0&&nav.ng==0);
	n=readrnx(file2,1,&obs,&nav);
		assert(n==0&&nav.ng>0);
	dumpgeph(nav.geph,nav.ng);

	printf("%s utest1 : OK\n",__FILE__);
}
/* geph2pos() */
void utest2(void)
{
	char *file[]={"brdc0910.09g"};
	obs_t obs={0};
	nav_t nav={0};
	obsd_t data={{0}};
	double rs[3],dts[1],ep[]={2009,4,1,0,0,0},tint=10.0,tspan=86400.0,tow;
	int i,n,prn=7,week;
	n=readrnx(file,1,&obs,&nav);
	
	for (i=0;i<tspan/tint;i++) {
		data.sat=satno(SYS_GLO,prn);
		data.time=timeadd(epoch2time(ep),tint*i);
		data.P[0]=1E-9;
		satpos(data.time,&data,1,&nav,rs,dts);
		tow=time2gpst(data.time,&week);
		printf("%4d %6.0f %2d %13.3f %13.3f %13.3f %10.5E\n",week,tow,data.sat,rs[0],rs[1],rs[2],dts[0]);
	}
	printf("\n");
}
/* readsp3() */
void utest3(void)
{
	char *file="igl15253.sp3";
	peph_t *peph;
	double tow,*pos;
	int i,n,week,prn=7,sat;
	
	sat=satno(SYS_GLO,prn);
	
	n=readsp3(file,&peph);
	
	for (i=0;i<n;i++) {
		tow=time2gpst(peph[i].time,&week);
		pos=peph[i].pos[sat-1];
		printf("%4d %6.0f %2d %13.3f %13.3f %13.3f %10.5E\n",
			   week,tow,sat,pos[0],pos[1],pos[2],pos[3]);
	}
	printf("\n");
}
/* readsap() */
void utest4(void)
{
	char *file="../../data/igs05.atx",id[32];
	double sap[MAXSAT][3],ep[]={2009,4,1,0,0,0};
	gtime_t time=epoch2time(ep);
	int i;
	
	readsap(file,time,sap);
	
	for (i=0;i<MAXSAT;i++) {
		satno2id(i+1,id);
		printf("%2d %-4s %8.3f %8.3f %8.3f\n",i+1,id,sap[i][0],sap[i][1],sap[i][2]);
	}
	printf("\n");
}
/* satposp() */
void utest5(void)
{
	char *file1[]={"brdc0910.09g"};
	char *file2="igl15253.sp3";
	char *file3="../../data/igs05.atx";
	double ep[]={2009,4,1,0,0,0};
	double rs1[3],dts1[1],rs2[3],dts2[1],dr[3],ddts,tint=10.0,tspan=86400.0,tow;
	gtime_t time=epoch2time(ep);
	obs_t obs={0};
	nav_t nav={0};
	obsd_t data={{0}};
	int i,j,nobs,week,prn=21;
	
	nobs=readrnx(file1,1,&obs,&nav);
	nav.ne=readsp3(file2,&nav.peph);
	readsap(file3,time,nav.sap);
	
	for (i=0;i<tspan/tint;i++) {
		data.sat=satno(SYS_GLO,prn);
		data.time=timeadd(epoch2time(ep),tint*i);
		data.P[0]=1E-9;
		tow=time2gpst(data.time,&week);
		satpos (data.time,&data,1,&nav,rs1,dts1);
		satposp(data.time,&data,1,&nav,rs2,dts2);
		if (norm(rs1,3)<=0.0||norm(rs2,3)<=0.0) continue;
		for (j=0;j<3;j++) dr[j]=rs1[j]-rs2[j];
		ddts=dts1[0]-dts2[0];
		printf("%4d %6.0f %2d %8.3f %8.3f %8.3f %10.5E\n",week,tow,data.sat,
			   dr[0],dr[1],dr[2],ddts);
		assert(norm(dr,3)<10.0);
	}
	printf("\n");
}
/* unit test main */
int main(int argc, char **argv)
{
	utest1();
	utest2();
	utest3();
	utest4();
	utest5();
	return 0;
}
